indexing
	description: "Features of IDENTITIES class"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_IDENTITIES_FEATURES

inherit
	CI_IDENTITIES_NAMES_CLIENT

	CI_PRINTABLE_CLASSES
		export {NONE}
			all
		end

	CI_CODE_GENERATION
		export {NONE}
			all
		end
create
	make

feature -- Initialization
	make (an_impl_class: like impl_class; a_universe: like universe) is
			-- Create identities features inside `an_impl_class' within `a_universe'
		require
			an_impl_class_exists: an_impl_class /= Void
			a_universe_exists: a_universe /= Void
		do
			universe := a_universe
			impl_class := an_impl_class
		ensure
			impl_class_set: impl_class = an_impl_class
			universe_set: universe = a_universe
		end

feature -- Access
	identity_function (a_class: ET_CLASS): ET_DO_FUNCTION is
			-- Identity function for instances of `a_class', defined inside `an_implementation_class' within `a_universe'
		require
			a_class_exists: a_class /= Void
		local
			attr: ET_ATTRIBUTE
			query_var: CI_QUERY
			locs: ET_LOCAL_VARIABLE_LIST
			l_id: ET_IDENTIFIER
			l: ET_LOCAL_VARIABLE
			body: ET_COMPOUND
			instr: ET_ASSIGNMENT
			field_by_name_call: ET_CALL_EXPRESSION
			field_by_name_args: ET_ACTUAL_ARGUMENT_LIST
			i: INTEGER
			uniform_object: LINKED_LIST [ET_EXPRESSION]
		do
			-- Create function signature
			create Result.make (Identities_names.identity_name (a_class.name),
				single_formal (create {ET_FORMAL_ARGUMENT}.make (Identities_names.object_argument, universe.any_type)),
				universe.integer_type,
				impl_class)

			-- Create locals and body
			create locs.make
			create instr.make (tokens.result_keyword, default_identity)
			create body.make
			body.force_first (instr)
			Result.set_compound (body)

			create uniform_object.make
			from
				i := 1
			until
				i > a_class.queries.count
			loop
				attr ?= a_class.queries.item (i)
				if attr /= Void then
					create query_var.make (attr, create {CI_CURRENT}.make (Void, a_class, universe))

					create field_by_name_args.make_with_capacity (2)
					field_by_name_args.put_first (manifest_string (attr.lower_name))
					field_by_name_args.put_first (Identities_names.object_argument)
					create field_by_name_call.make (Void, Identities_names.field_by_name_name, field_by_name_args)

					create l_id.make (attr.lower_name + "_field")
					create l.make (l_id, query_var.type.kind.uniform_identity_argument_type)
					body.force_first (create {ET_ASSIGNMENT_ATTEMPT}.make (l_id, field_by_name_call))

					locs.force_first (l)

					uniform_object.extend (query_var.type.kind.uniform_identity (l_id))
				end
				i := i + 1
			end

			Result.set_locals (locs)
			instr.set_source (folding (uniform_object))
		ensure
			result_exists: Result /= Void
		end

	unknown_identity_function (an_expanded_classes: LIST [ET_CLASS_NAME]): ET_DO_FUNCTION is
			-- Identity function for unknown type in system where classes from `an_expanded_classes' are expanded
			-- or have expanded entities
		require
			an_expanded_classes_exists: an_expanded_classes /= Void
		local
			p_type: CI_PRINTABLE
			r_type: CI_REFERENCE
			h_id, c_id, t_id: ET_IDENTIFIER
			locs: ET_LOCAL_VARIABLE_LIST
			body, compound: ET_COMPOUND
			if_instr: ET_IF_INSTRUCTION
			elseif_part: ET_ELSEIF_PART
			elseif_parts: ET_ELSEIF_PART_LIST
		do
			-- Create function signature
			create Result.make (Identities_names.unknown_identity_name,
				single_formal (create {ET_FORMAL_ARGUMENT}.make (Identities_names.object_argument, universe.any_type)),
				universe.integer_type,
				impl_class)

			-- Create locals
			create h_id.make ("h")
			create c_id.make ("c")
			create t_id.make ("t")

			create p_type.make (universe)
			create r_type.make (universe)
			create locs.make_with_capacity (3)
			locs.put_first (create {ET_LOCAL_VARIABLE}.make (t_id, universe.string_type))
			locs.put_first (create {ET_LOCAL_VARIABLE}.make (c_id, universe.string_type))
			locs.put_first (create {ET_LOCAL_VARIABLE}.make (h_id, p_type.uniform_identity_argument_type))
			Result.set_locals (locs)

			-- Create body
			create body.make_with_capacity (3)

			create compound.make_with_capacity (2)
			compound.set_keyword (tokens.then_keyword)
			compound.put_first (create {ET_ASSIGNMENT}.make (tokens.result_keyword, p_type.uniform_identity (h_id)))
			compound.put_first (create {ET_ASSIGNMENT_ATTEMPT}.make (h_id, Identities_names.object_argument))
			create if_instr.make (create {ET_CALL_EXPRESSION}.make (Void, Identities_names.is_printable_name, single_actual (c_id)),
				compound)

			if an_expanded_classes.is_empty then
				create compound.make
			else
				compound := single_instruction (expanded_test (c_id, an_expanded_classes))
			end
			create elseif_part.make (create {ET_INFIX_EXPRESSION}.make (expanded_mark_test (t_id),
					tokens.or_keyword,
					create {ET_CALL_EXPRESSION}.make (Void, Identities_names.is_expanded_name, single_actual (c_id))),
				compound)
			elseif_part.then_compound.set_keyword (tokens.then_keyword)
			create elseif_parts.make_with_capacity (1)
			elseif_parts.put_first (elseif_part)
			if_instr.set_elseif_parts (elseif_parts)

			create compound.make_with_capacity (2)
			compound.put_first (create {ET_ASSIGNMENT}.make (tokens.result_keyword,
				r_type.uniform_identity (Identities_names.object_argument)))
			compound.set_keyword (tokens.else_keyword)
			if_instr.set_else_compound (compound)

			body.put_first (if_instr)
			body.put_first (create {ET_ASSIGNMENT}.make (t_id,
				create {ET_CALL_EXPRESSION}.make (Void, tokens.type_name_feature_name, single_actual (Identities_names.object_argument))))
			body.put_first (create {ET_ASSIGNMENT}.make (c_id,
				create {ET_CALL_EXPRESSION}.make (Void, tokens.class_name_feature_name, single_actual (Identities_names.object_argument))))

			create if_instr.make (create {ET_EQUALITY_EXPRESSION}.make (Identities_names.object_argument, create {ET_SYMBOL}.make_not_equal,
				tokens.void_keyword), body)
			if_instr.set_else_compound (single_instruction (create {ET_ASSIGNMENT}.make (tokens.result_keyword, manifest_integer ("0"))))
			if_instr.then_compound.set_keyword (tokens.then_keyword)
			if_instr.else_compound.set_keyword (tokens.else_keyword)

			Result.set_compound (single_instruction (if_instr))
		ensure
			result_exiats: Result /= Void
		end

	printable_hash_code_function: ET_DO_FUNCTION is
			-- Function that returns proper hashcode for printable variables
		local
			if_instr: ET_IF_INSTRUCTION
		do
			-- Create function signature
			create Result.make (Identities_names.printable_hash_code_name,
				single_formal (create {ET_FORMAL_ARGUMENT}.make (Identities_names.item_argument, type_from_nongeneric_class (universe.hashable_class))),
				universe.integer_type,
				impl_class)

			-- Create body
			create if_instr.make (
				create {ET_CALL_EXPRESSION}.make (Identities_names.item_argument, tokens.is_hashable_feature_name, Void),
				single_instruction (create {ET_ASSIGNMENT}.make (tokens.result_keyword,
					create {ET_CALL_EXPRESSION}.make (Identities_names.item_argument, tokens.hash_code_feature_name, Void))))
			if_instr.then_compound.set_keyword (tokens.then_keyword)
			if_instr.set_else_compound (single_instruction (create {ET_ASSIGNMENT}.make (tokens.result_keyword, manifest_integer ("-1"))))
			if_instr.else_compound.set_keyword (tokens.else_keyword)
			Result.set_compound (single_instruction (if_instr))
		ensure
			result_exiats: Result /= Void
		end

	is_expanded_function (an_expanded_classes: LIST [ET_CLASS_NAME]): ET_DO_FUNCTION is
			-- Boolean function with string argument that says, if its argument is a name of expanded class
		require
			an_expanded_classes_exists: an_expanded_classes /= Void
		do
			Result := class_name_test (Identities_names.is_expanded_name, an_expanded_classes)
		ensure
			correct_name: Result.name.same_feature_name (Identities_names.is_expanded_name)
		end

	is_printable_function: ET_DO_FUNCTION is
			-- Boolean function with string argument that says, if its argument is a name of printable class
		do
			Result := class_name_test (Identities_names.is_printable_name, printable_classes)
		ensure
			correct_name: Result.name.same_feature_name (Identities_names.is_printable_name)
		end

	field_by_name_function: ET_DO_FUNCTION is
			-- Function with two arguments (`obj: ANY' and `field_name: STRING') that returns field of object `obj'
			-- with name `field_feature_name'
		local
			i: ET_IDENTIFIER
			locs: ET_LOCAL_VARIABLE_LIST
			args: ET_FORMAL_ARGUMENT_LIST
			loop_instr: ET_LOOP_INSTRUCTION
			if_instr, inner_if: ET_IF_INSTRUCTION
			elseif_parts: ET_ELSEIF_PART_LIST
			ass: ET_ASSIGNMENT
			actual_args: ET_ACTUAL_ARGUMENT_LIST
			field_count_call, field_name_call, is_equal_call, field_call: ET_CALL_EXPRESSION
			loop_body, if_body: ET_COMPOUND
			k: INTEGER
		do
			-- Create function signature
			create args.make_with_capacity (2)
			args.put_first (create {ET_FORMAL_ARGUMENT}.make (Identities_names.field_name_argument, universe.string_type))
			args.put_first (create {ET_FORMAL_ARGUMENT}.make (Identities_names.object_argument, universe.any_type))

			create Result.make (Identities_names.field_by_name_name, args, universe.any_type, impl_class)

			-- Create locals
			create locs.make_with_capacity (printable_class_refs_count + 1)
			from
				printable_class_refs.finish
				printable_classes.finish
			until
				printable_class_refs.before
			loop
				if printable_class_refs.item /= Void then
					locs.put_first (create {ET_LOCAL_VARIABLE}.make (
						create {ET_IDENTIFIER}.make (printable_classes.item.lower_name + "_ref"),
						type_from_nongeneric_class (universe.eiffel_class (printable_class_refs.item))))
				end
				printable_class_refs.back
				printable_classes.back
			end
			create i.make ("i")
			locs.put_first (create {ET_LOCAL_VARIABLE}.make (i, universe.integer_type))
			Result.set_locals (locs)

			-- Create function body
			create loop_body.make_with_capacity (2)
			loop_body.set_keyword (tokens.loop_keyword)
			create ass.make (i,
				create {ET_INFIX_EXPRESSION}.make (i, create {ET_SYMBOL_OPERATOR}.make_plus, manifest_integer ("1")))
			loop_body.put_first (ass)

			create actual_args.make_with_capacity (2)
			actual_args.put_first (Identities_names.object_argument)
			actual_args.put_first (i)
			create field_name_call.make (Void, tokens.field_name_feature_name, actual_args)
			create is_equal_call.make (field_name_call, tokens.is_equal_feature_name,
				single_actual (Identities_names.field_name_argument))

			create if_body.make_with_capacity (3)
			if_body.set_keyword (tokens.then_keyword)
			create field_count_call.make (Void, tokens.field_count_feature_name, single_actual (Identities_names.object_argument))
			create ass.make (i, field_count_call)
			if_body.put_first (ass)

			if printable_class_refs_count > 0 then
				from
					printable_class_indices.finish
					k := locs.count
				until
					printable_class_indices.before
				loop
					if printable_class_indices.item /= Void then
						if inner_if = Void then
							create inner_if.make (
								resolve_condition (i, printable_class_indices.item),
								resolve_compound (locs.item (k).name))
							inner_if.then_compound.set_keyword (tokens.then_keyword)
							create elseif_parts.make_with_capacity (printable_class_refs_count - 1)
						else
							elseif_parts.put_first (create {ET_ELSEIF_PART}.make (
									resolve_condition (i, printable_class_indices.item),
									resolve_compound (locs.item (k).name)))
							elseif_parts.first.then_compound.set_keyword (tokens.then_keyword)
						end
						k := k - 1
					end
					printable_class_indices.back
				end
				inner_if.set_elseif_parts (elseif_parts)
				if_body.put_first (inner_if)
			end

			create field_call.make (Void, tokens.field_feature_name, actual_args)
			create ass.make (tokens.result_keyword, field_call)
			if_body.put_first (ass)

			create if_instr.make (is_equal_call, if_body)
			loop_body.put_first (if_instr)

			create loop_instr.make (single_instruction (create {ET_ASSIGNMENT}.make (i, manifest_integer ("1"))),
				create {ET_INFIX_EXPRESSION}.make (i, create {ET_SYMBOL_OPERATOR}.make_gt, field_count_call),
				loop_body)
			loop_instr.from_compound.set_keyword (tokens.from_keyword)

			Result.set_compound (single_instruction (loop_instr))
		ensure
			correct_name: Result.name.same_feature_name (Identities_names.field_by_name_name)
		end

feature {NONE} -- Implementation
	universe: ET_UNIVERSE
			-- Surrounding universe

	impl_class: ET_CLASS
			-- Features implementation class

	expanded_test (a_class_name: ET_IDENTIFIER; an_expanded_classes: LIST [ET_CLASS_NAME]): ET_IF_INSTRUCTION is
			-- If instruction: for all <class_name> from `an_expanded_classes'
			-- if `a_class_name'.is_equal ("<class_name>") then Result := <class_name>_identity elseif ... end
		require
			a_class_name_exists: a_class_name /= Void
			an_expanded_classes_exists: an_expanded_classes /= Void
			an_expanded_classes_not_empty: not an_expanded_classes.is_empty
		local
			e_type: CI_EXPANDED
			elseif_part: ET_ELSEIF_PART
			elseif_parts: ET_ELSEIF_PART_LIST
			is_equal_call: ET_CALL_EXPRESSION
			instr: ET_ASSIGNMENT
		do
			from
				an_expanded_classes.start
				create is_equal_call.make (a_class_name, tokens.is_equal_feature_name,
					single_actual (manifest_string (an_expanded_classes.item.upper_name)))
				create e_type.make (an_expanded_classes.item, tokens.unknown_class_name, universe)
				create instr.make (tokens.result_keyword, e_type.uniform_identity (Identities_names.object_argument))
				create Result.make (is_equal_call, single_instruction (instr))
				Result.then_compound.set_keyword (tokens.then_keyword)
				if an_expanded_classes.count > 1 then
					create elseif_parts.make_with_capacity (an_expanded_classes.count - 1)
				end
				an_expanded_classes.forth
			until
				an_expanded_classes.after
			loop
				create is_equal_call.make (a_class_name, tokens.is_equal_feature_name,
					single_actual (manifest_string (an_expanded_classes.item.upper_name)))
				e_type.set_class_name (an_expanded_classes.item)
				create instr.make (tokens.result_keyword, e_type.uniform_identity (Identities_names.object_argument))
				create elseif_part.make (is_equal_call, single_instruction (instr))
				elseif_part.then_compound.set_keyword (tokens.then_keyword)
				elseif_parts.put_first (elseif_part)
				an_expanded_classes.forth
			end
			Result.set_elseif_parts (elseif_parts)
		ensure
			result_exists: Result /= Void
		end

	class_name_test (a_name: ET_FEATURE_NAME; a_class_list: LIST [ET_CLASS_NAME]): ET_DO_FUNCTION is
			-- Boolean function with name `a_name' and a string argument
			-- that tests if its argument is equal to name of one of classes in `a_class_list'
		require
			a_name_exists: a_name /= Void
			a_class_list_exists: a_class_list /= Void
		local
			disjunct: ET_CALL_EXPRESSION
			res: ET_EXPRESSION
			instr: ET_ASSIGNMENT
		do
			-- Create function signature
			create Result.make (a_name,
				single_formal (create {ET_FORMAL_ARGUMENT}.make (Identities_names.class_name_argument, universe.string_type)),
				type_from_nongeneric_class (universe.boolean_class),
				impl_class)

			-- Create function body
			if not a_class_list.is_empty then
				from
					a_class_list.start
					create disjunct.make (Identities_names.class_name_argument,
						tokens.is_equal_feature_name,
						single_actual (manifest_string (a_class_list.item.upper_name)))
					res := disjunct
					a_class_list.forth
				until
					a_class_list.after
				loop
					create disjunct.make (Identities_names.class_name_argument,
						tokens.is_equal_feature_name,
						single_actual (manifest_string (a_class_list.item.upper_name)))

					create {ET_INFIX_EXPRESSION} res.make (res, tokens.or_keyword, disjunct)
					a_class_list.forth
				end
				create instr.make (tokens.result_keyword, res)
				Result.set_compound (single_instruction (instr))
			end
			ensure
				result_exists: Result /= Void
		end

	expanded_mark_test (type_name: ET_IDENTIFIER): ET_EXPRESSION is
			-- Boolean expression that tests, if `type_name' is a string representation of type with expanded mark
		require
			type_name_exists: type_name /= Void
		do
			create {ET_CALL_EXPRESSION} Result.make (type_name, tokens.has_substring_feature_name,
				single_actual (manifest_string ("expanded")))
		ensure
			reault_exists: Result /= Void
		end

	field_type_call (an_i: ET_EXPRESSION): ET_CALL_EXPRESSION is
			-- Call to `field_type' function with arguments `an_i' and `Identities_names.object_argument'
		local
			args: ET_ACTUAL_ARGUMENT_LIST
		do
			create args.make_with_capacity (2)
			args.put_first (Identities_names.object_argument)
			args.put_first (an_i)
			create Result.make (Void, tokens.field_type_feature_name, args)
		end

	resolve_condition (an_i: ET_EXPRESSION; a_printable_class_index: ET_IDENTIFIER): ET_EQUALITY_EXPRESSION is
			-- Boolean expression that sais, if type of `i'th field is `printable_class_index'
		do
			create Result.make (field_type_call (an_i), create {ET_SYMBOL}.make_equal, a_printable_class_index)
		end

	resolve_compound (a_ref: ET_IDENTIFIER): ET_COMPOUND is
			-- Compound that refines type of `field_by_name' result with the help of printable type reference `a_ref'
		do
			create Result.make_with_capacity (2)
			Result.put_first (create {ET_ASSIGNMENT}.make (
				tokens.result_keyword,
				create {ET_CALL_EXPRESSION}.make (a_ref, tokens.item_feature_name, Void)))
			Result.put_first (create {ET_ASSIGNMENT_ATTEMPT}.make (a_ref, tokens.result_keyword))
		end

feature {NONE} -- Folding
	folding (an_object: LIST [ET_EXPRESSION]): ET_EXPRESSION is
			-- Folding of `an_object' items;
			-- used to obtain identities of composite objects; by default folding function is:
			-- sum [i = 1.. an_object.count] cyclic_left_shift (an_object [i], 32 // an_object.count * (i - 1)))
		require
			an_object_exists: an_object /= Void
		do
			if an_object.is_empty then
				Result := default_identity
			else
				from
					an_object.start
					Result := folding_component (an_object.item, an_object.index, an_object.count)
					an_object.forth
				until
					an_object.after
				loop
					create {ET_CALL_EXPRESSION} Result.make (Result, tokens.bit_xor_feature_name,
						single_actual (folding_component (an_object.item, an_object.index, an_object.count)))
					an_object.forth
				end
			end
		ensure
			result_exists: Result /= Void
			is_default_for_empty_object: an_object.is_empty implies Result = default_identity
		end

	folding_component (a_field: ET_EXPRESSION; i, n: INTEGER): ET_EXPRESSION is
			-- Component in folding function
		require
			a_field_exists: a_field /= Void
			n_positive: n > 0
			i_positive: i > 0
			i_not_too_large: i <= n
		local
			bits_count: INTEGER
			left_shifted, right_shifted: ET_CALL_EXPRESSION
		do
			bits_count := {CI_VARIABLE_TYPE_KIND}.uniform_identity_type_size // n * (i - 1)
			create left_shifted.make (a_field, tokens.bit_shift_left_feature_name,
				single_actual (manifest_integer (bits_count.out)))
			create right_shifted.make (a_field, tokens.bit_shift_right_feature_name,
				single_actual (manifest_integer (({CI_VARIABLE_TYPE_KIND}.uniform_identity_type_size - bits_count).out)))
			create {ET_CALL_EXPRESSION} Result.make (left_shifted, tokens.bit_xor_feature_name, single_actual (right_shifted))
		ensure
			result_exists: Result /= Void
		end

	default_identity: ET_EXPRESSION is
			-- Default identity for object, which don't have one (e.g., don't have fields)
		once
			Result := manifest_integer ("0")
		end

invariant
	universe_exists: universe /= Void
	impl_class_exists: impl_class /= Void
	default_identity_exists: default_identity /= Void
end
