indexing
	description: "Objects that generate special class (called 'IDENTITIES') in instrumented system %
		%that contains an identity function for every expanded class in original system %
		%(or class that has expanded instances - this won't be needed with standard Eiffel) %
		% and one more identity function for entities of unknown type"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_IDENTITIES_GENERATOR

inherit
	ET_AST_ITERATOR
		redefine
			make,
			process_class,
			process_class_type
		end

	CI_PRINTABLE_CLASSES
		export {NONE}
			all
		end

	CI_IDENTITIES_NAMES_CLIENT

	CI_SETTINGS_CLIENT

	CI_CODE_GENERATION
		export {NONE}
			all
		end
create
	make

feature -- Initialization
	make (a_universe: like universe) is
			-- Create identities generator for `a_universe'
		do
			Precursor (a_universe)
			create expanded_classes.make
			create expandedly_used_classes.make
		end

feature -- Basic operations
	generate_identities is
			-- Generate IDENTITIES class
		local
			features: CI_IDENTITIES_FEATURES
			queries: ET_QUERY_LIST
			f: ET_QUERY
		do
			create_class
			process_system
			create features.make (identities, universe)
			create queries.make_with_capacity (expandedly_used_classes.count + 5)

			from
				expandedly_used_classes.start
			until
				expandedly_used_classes.after
			loop
				f := features.identity_function (universe.eiffel_class (expandedly_used_classes.item))
				f.set_feature_clause (identity_clause)
				queries.put_first (f)
				expandedly_used_classes.forth
			end

			f := features.unknown_identity_function (expandedly_used_classes)
			f.set_feature_clause (identity_clause)
			queries.put_first (f)

			f := features.field_by_name_function
			f.set_feature_clause (implementation_clause)
			queries.put_first (f)

			f := features.is_printable_function
			f.set_feature_clause (implementation_clause)
			queries.put_first (f)

			f := features.is_expanded_function (expanded_classes)
			f.set_feature_clause (implementation_clause)
			queries.put_first (f)

			f := features.printable_hash_code_function
			f.set_feature_clause (implementation_clause)
			queries.put_first (f)

			identities.set_queries (queries)
		ensure
			identities_exists: identities /= Void
			identities_name_correct: identities.name.same_class_name (Identities_names.identities_name)
			identities_in_universe: universe.has_class (Identities_names.identities_name)
		end

feature -- Access
	identities: ET_CLASS
			-- IDENTITIES class

feature {ET_AST_NODE} -- Processing
	process_class (a_class: ET_CLASS) is
			-- Process `a_class'
		do
			Precursor (a_class)
			if a_class.is_expanded and not is_printable (a_class) then -- Printable classes are not considered expanded
				expanded_classes.extend (a_class.name)
				expandedly_used_classes.extend (a_class.name)
			end
		end

	process_class_type (a_type: ET_CLASS_TYPE) is
			-- Process `a_type'
		local
			c: ET_CLASS
		do
			if a_type.type_mark /= Void and then a_type.type_mark.text.is_equal (tokens.expanded_keyword.text) then
				c := universe.eiffel_class (a_type.name)
			 	if not c.is_expanded and not is_printable (c) then
			 		expandedly_used_classes.extend (c.name)
			 	end
			end
		end

feature {NONE} -- Implementation		
	expanded_classes: LINKED_LIST [ET_CLASS_NAME]
			-- All expanded nonprintable classes in current system

	expandedly_used_classes: LINKED_LIST [ET_CLASS_NAME]
			-- All nonprintable classes in current system, which are expanded or have instances marked "expanded"

	create_class is
			-- Create identities class with valid name and 'inherit' clause
		local
			parent: ET_CLASS_TYPE
			parents: ET_PARENT_LIST
		do
			Identities_names.generate_identities_name (universe)
			create identities.make (Identities_names.identities_name, universe.classes.count + 1)
			universe.classes.force_new (identities, identities.name)
			identities.set_group (Settings.instrumented_cluster)
			identities.set_parsed

			create parent.make (Void, Identities_names.parent_name, universe.eiffel_class (Identities_names.parent_name))
			create parents.make_with_capacity (1)
			parents.put_first (create {ET_PARENT}.make (parent, Void, Void, Void, Void, Void))
			identities.set_parents (parents)

			identities.set_feature_clauses (feature_clauses)
		end

	identity_clause : ET_FEATURE_CLAUSE is
			-- Feature clause for `<class_name>_identity' and `unknown_identity' functions
		once
			create Result.make (clients_any)
			add_comment ("Identity", Result)
		end

	implementation_clause: ET_FEATURE_CLAUSE is
			-- Feature clause for auxiliary features
		once
			create Result.make (clients_none)
			add_comment ("Implementation", Result)
		end

	feature_clauses: ET_FEATURE_CLAUSE_LIST is
			-- IDENTITIES feature clauses
		once
			create Result.make_with_capacity (2)
			Result.put_first (identity_clause)
			Result.put_first (implementation_clause)
		end

	process_system is
			-- Process all classes in system and fill `expanded_classes' and `expanedly_used_classes' lists
		local
			c: ET_CLASS
		do
			from
				universe.classes.start
			until
				universe.classes.after
			loop
				c := universe.classes.item_for_iteration
				if c.in_system then
					c.process (Current)
				end
				universe.classes.forth
			end
		end

invariant
	expanded_classes_exists: expanded_classes /= Void
	expandedly_esed_classes_exists: expandedly_used_classes /= Void
	identity_clause_exists: identity_clause /= Void
	implementation_clause_exists: implementation_clause /= Void
	feature_clauses_exists: feature_clauses /= Void
end
