indexing
	description: "Objects that insert inferred assertions into source code"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_ANNOTATOR

inherit
	CI_PROGRAM_POINT_FINDER
		rename
			error_handler as processor_error_handler
		export {NONE}
			processor_error_handler
		redefine
			make,
			process_program_point
		end

	CI_ERROR_HANDLER_CLIENT
		rename
			make_null as error_handler_make_null
		end

	CI_SETTINGS_CLIENT

	KL_SHARED_FILE_SYSTEM

create
	make

feature -- Initialization
	make (a_universe: like universe; a_ppt_printer: like printer) is
			-- Create annotator in `a_universe' with program point information defined by `a_ppt_printer'
		do
			Precursor (a_universe, a_ppt_printer)
			error_handler_make_null
			from
				Settings.classes_to_instrument.start
			until
				Settings.classes_to_instrument.after
			loop
				universe.eiffel_class (Settings.classes_to_instrument.item_for_iteration).process (Current)
				Settings.classes_to_instrument.forth
			end
		end

feature -- Access
	annotated_classes: DS_HASH_SET [ET_CLASS]
			-- Classes that have been annotated			

feature -- Basic operations
	read_assertions is
			-- Read inferred assertions from `a_file' into `program_points'
		local
			current_ppt: CI_PROGRAM_POINT
			input_file: KI_TEXT_INPUT_FILE
			str: STRING
		do
			input_file := file_system.new_input_file (Settings.invariants_name)
			input_file.open_read
			if input_file.is_open_read then
				from
				until
					input_file.end_of_input
				loop
					input_file.read_line
					if not input_file.last_string.is_empty then
						if input_file.last_string.item (1) = program_point_separator then
							-- New program point begins
							input_file.read_line
							str := input_file.last_string.string
							if str.ends_with ({CI_DAIKON_PROGRAM_POINT_PRINTER}.aggregative_exit_kind) then
								-- In Daikon output, exit points are renamed into aggregative exit points
								str.append ({CI_DAIKON_PROGRAM_POINT_PRINTER}.default_exit_id)
							end
							program_points.search (str)
							if program_points.found then
								current_ppt := program_points.found_item
							else
								error_handler.report_ppt_not_found (str)
							end
						elseif current_ppt = Void then
							error_handler.report_error_message ("File with inferred assertions " + input_file.name +
								" has invalid format")
						else
							-- Previous program point continues
							current_ppt.inferred_assertions.extend (create {CI_INFERRED_ASSERTION}.make (input_file.last_string.string))
						end
					end
				end
				input_file.close
			else
				error_handler.report_error_message ("Can't open file with inferred assertions: " + input_file.name)
			end
		end

	annotate_classes is
			-- Annotate classes with inferred assertions
		local
			ppt: CI_PROGRAM_POINT
			tracker: CI_NONSENSICAL_TRACKER
		do
			create tracker.make (program_points)
			tracker.set_error_handler (error_handler)
			tracker.read_trace
			create annotated_classes.make_default
			create guarded_program_points.make_equal (program_points.count)
			from
				program_points.start
			until
				program_points.after
			loop
				ppt := program_points.item_for_iteration
				remove_uninteresting_assertions (ppt)
				guard_assertions (ppt, program_points.key_for_iteration)
				remove_suppressed_assertions (ppt)
				replace_variable_names (ppt)
				ppt.annotate
				if not ppt.inferred_assertions.is_empty then
					annotated_classes.force (ppt.surrounding_class)
				end
				program_points.forth
			end
		ensure
			annotated_classes_exists: annotated_classes /= Void
		end

	print_annotated_classes is
			-- Print classes from `annotated_classes'
		require
			annotated_classes_exists: annotated_classes /= Void
		do
			from
				annotated_classes.start
			until
				annotated_classes.after
			loop
				print_class (annotated_classes.item_for_iteration)
				annotated_classes.forth
			end
		end

feature {NONE} -- Implementation
	guarded_program_points: DS_HASH_SET [STRING]

	program_point_separator: CHARACTER is '='
			-- First character of line separating program points in input file

	process_program_point (a_ppt: CI_PROGRAM_POINT) is
			-- Call precursor, then flatten variables
		do
			Precursor (a_ppt)
			flatten_variables (a_ppt)
		end

	remove_suppressed_assertions (a_ppt: CI_PROGRAM_POINT) is
			-- Remove inferred assertions at `a_ptt' that are equal to assertions at its suppressor
		require
			a_ppt_exists: a_ppt /= Void
		local
			s_name: STRING
			s: CI_PROGRAM_POINT
		do
			if Settings.suppression_enabled and a_ppt.suppressor /= Void then
				s_name := printer.program_point_name (a_ppt.suppressor)
				s := program_points.item (s_name)
				guard_assertions (s, s_name)
				from
					a_ppt.inferred_assertions.start
				until
					a_ppt.inferred_assertions.after
				loop
					if s.inferred_assertions.has (a_ppt.inferred_assertions.item) then
						a_ppt.inferred_assertions.remove
					else
						a_ppt.inferred_assertions.forth
					end
				end
			end
		end

	remove_uninteresting_assertions (a_ppt: CI_PROGRAM_POINT) is
			-- Remove inferred assertions at `a_ptt' that are considered uninteresting
		require
			a_ppt_exists: a_ppt /= Void
		do
			from
				a_ppt.inferred_assertions.start
			until
				a_ppt.inferred_assertions.after
			loop
				if Settings.is_uninteresting (a_ppt.inferred_assertions.item) then
					a_ppt.inferred_assertions.remove
				else
					a_ppt.inferred_assertions.forth
				end
			end
		end

	guard_assertions (a_ppt: CI_PROGRAM_POINT; a_ppt_name: STRING) is
			-- Guard assertions at `a_ppt' (called `a_ppt_name') with preconditions
		require
			a_ppt_exists: a_ppt /= Void
			guarded_program_points_exists: guarded_program_points /= Void
		do
			if not guarded_program_points.has (a_ppt_name) then
				from
					a_ppt.variables.start
				until
					a_ppt.variables.after
				loop
					if a_ppt.variables.item.was_nonsensical then
						guard_with_variable (a_ppt, a_ppt.variables.item)
					end
					a_ppt.variables.forth
				end
				guarded_program_points.put_new (a_ppt_name)
			end
		end

	replace_variable_names (a_ppt: CI_PROGRAM_POINT) is
			-- Replace Daikon variable names with Eiffel names
		require
			a_ppt_exists: a_ppt /= Void
		do
			from
				a_ppt.inferred_assertions.start
			until
				a_ppt.inferred_assertions.after
			loop
				a_ppt.inferred_assertions.item.replace_string_values
				a_ppt.inferred_assertions.forth
			end
		end

	guard_with_variable (a_ppt: CI_PROGRAM_POINT; a_variable: CI_VARIABLE) is
			-- Guard assertions at `a_ppt' with evaluation preconditions of `a_variable'
		require
			a_ppt_exists: a_ppt /= Void
			a_variable_exists: a_variable /= Void
		do
			from
				a_ppt.inferred_assertions.start
			until
				a_ppt.inferred_assertions.after
			loop
				if a_ppt.inferred_assertions.item.has_variable (a_variable.eiffel_name) then
					if a_variable.has_watched_once then
						a_ppt.inferred_assertions.remove
					else
						a_ppt.inferred_assertions.item.add_preconditions (a_variable.absolute_preconditions)
						a_ppt.inferred_assertions.forth
					end
				else
					a_ppt.inferred_assertions.forth
				end
			end
		end

	flatten_variables (a_ppt: CI_PROGRAM_POINT) is
			-- Find all variables at `a_ppt', unfold them and add unfolded forms to set of variables
		local
			s: TWO_WAY_SORTED_SET [CI_VARIABLE]
		do
			if not a_ppt.variables_added then
				a_ppt.add_variables
			end
			create s.make
			s.compare_objects
			from
				a_ppt.variables.start
			until
				a_ppt.variables.after
			loop
				a_ppt.variables.item.set_instrumentation_mode (False)
				flatten_variable (a_ppt.variables.item.unfolded, s)
				a_ppt.variables.forth
			end
			a_ppt.variables.merge (s)
		end

	flatten_variable (an_unfolded_variable: TREE [CI_VARIABLE]; a_set: TWO_WAY_SORTED_SET [CI_VARIABLE]) is
			-- Put all variables from `an_unfolded_variable' into `a_set'
		require
			an_unfolded_variable_exists: an_unfolded_variable /= Void
			a_set_exists: a_set /= Void
		do
			a_set.put (an_unfolded_variable.item)
			from
				an_unfolded_variable.child_start
			until
				an_unfolded_variable.child_after
			loop
				flatten_variable (an_unfolded_variable.child, a_set)
				an_unfolded_variable.child_forth
			end
		end

	print_class (a_class: ET_CLASS) is
			-- Print `a_class' to file
		require
			a_class_exists: a_class /= Void
		local
			out_file: KI_TEXT_OUTPUT_FILE
			class_printer: CI_AST_PRETTY_PRINTER
		do
			out_file := file_system.new_output_file (Settings.annotated_class_file_name (a_class.name))
			out_file.open_write
			if out_file.is_open_write then
				create class_printer.make (out_file, universe)
				a_class.process (class_printer)
			else
				error_handler.report_error_message ("Can't open file " + Settings.annotated_class_file_name (a_class.name))
			end
			out_file.close
		end
end
