indexing
	description: "Program points at entry to creation procedure (associated with preconditions)"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_CREATION_PROCEDURE_ENTRY

inherit
	CI_ROUTINE_ENTRY
		rename
			make as make_with_routine_and_class
		export {NONE}
			make_with_routine_and_class
		redefine
			suppressor,
			surrounding_routine,
			add_variables
		end

create
	make

feature -- Initialization
	make (a_routine: like surrounding_routine; a_clients: like creator_clients; a_class: like surrounding_class;
		a_universe: like universe) is
			-- Create a program point within creation procedure `a_routine' that is available for creation to `a_clients'
			-- in `a_class' within `a_universe'
		require
			a_routine_exists: a_class /= Void
			a_clients_exists: a_clients /= Void
			a_class_exists: a_class /= Void
			a_universe_exists: a_universe /= Void
		do
			creator_clients := a_clients
			make_with_routine_and_class (a_routine, a_class, a_universe)
		ensure
			surrounding_routine_set: surrounding_routine = a_routine
			clients_set: creator_clients = a_clients
			surrounding_class_set: surrounding_class = a_class
			universe_set: universe = a_universe
		end

feature -- Access
	surrounding_routine: ET_PROCEDURE
			-- Surrounding creation procedure

	creator_clients: ET_CLASS_NAME_LIST
			-- Classes to which `surrounding_routine' is available for creation

	suppressor: CI_PROGRAM_POINT is
			-- Program point, who's assertions suppress assertions at current program point (Void if no suppressor)
		do
		end

feature -- Variables
	add_variables is
			-- Find variables in scope and store them into `variables'
		local
			cur: CI_CURRENT
		do
			Precursor
			-- Remove "Current" from variables list
			create cur.make (surrounding_routine, surrounding_class, universe)
			variables.prune (cur)
			-- Add routine's creator clients to all variable's `clients'
			-- Is it correct that names in creation procedure precondition should be exported to its clients and creator clients?
			from
				variables.start
			until
				variables.after
			loop
				variables.item.clients.resize (variables.item.clients.count + creator_clients.count)
				variables.item.clients.extend_first (creator_clients)
				variables.forth
			end
			variables_added := True
		end

invariant
	creator_clients_exists: creator_clients /= Void
end
