indexing
	description: "Output handlers for CITADEL"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_ERROR_HANDLER

inherit
	AP_ERROR_HANDLER
		redefine
			report_error_message,
			report_warning_message
		end

	CI_SETTINGS_CLIENT

	KL_SHARED_EXCEPTIONS

create
	make_standard,
	make_null

feature -- Status setting
	enable_verbose is
			-- Set `info_file' to standart output
		do
			info_file := std.output
		ensure
			verbose: is_verbose
		end

	disable_verbose is
			-- Set `info_file' to null output stream
		do
			info_file := null_output_stream
		ensure
			not_verbose: not is_verbose
		end

	terminate is
			-- Terminate execution
		do
			report_exiting
			io.read_character
			Exceptions.die (1)
		end

feature -- Reporting info
	report_version is
			-- Report version message
		do
			report_info_message ("CITADEL version " + version_number)
		end

	report_start_up is
			-- Report message on start
		do
			report_info_message ("This is CITADEL")
		end

	report_initial_system_compilation is
			-- Report message on starting initial system compilation
		do
			report_info_message ("Compiling initial system")
		end

	report_instrumentation is
			-- Report message on starting instrumentation
		do
			report_info_message ("Instrumenting system")
		end

	report_instrumenting_class (a_class_name: ET_CLASS_NAME) is
			-- Report that class called `a_class_name' is being instrumented
		do
			report_info_message ("Instrumenting class " + a_class_name.upper_name)
		end

	report_instrumenting_routine (a_name: ET_FEATURE_NAME) is
			-- Report that class routine called `a_name' is being instrumented
		do
			report_info_message ("Instrumenting routine " + a_name.lower_name)
		end

	report_compilation is
			-- Report message on starting instrumented system compilation
		do
			report_info_message ("Compiling instrumented system")
		end

	report_execution is
			-- Report message on starting instrumented system execution
		do
			report_info_message ("Executing instrumented system")
		end

	report_inference is
			-- Report message on starting assertion inference
		do
			report_info_message ("Inferring invariants")
		end

	report_printing is
			-- Report message on starting invariants printing
		do
			report_info_message ("Printing invariants to file " + Settings.invariants_name)
		end

	report_annotation is
			-- Report message on starting system annotation
		do
			report_info_message ("Annotating system with assertions")
		end

	report_exiting is
			-- Report message before exiting
		do
			report_info_message ("Exiting CITADEL")
		end

feature -- Reporting errors
	report_error_message (a_message: STRING) is
			-- Report `a_message'
		do
			error_file.put_string ("ERROR: ")
			error_file.put_string (a_message)
			error_file.put_new_line
			has_error := True
		ensure then
			has_error: has_error
		end

	report_missing_ecf_filename_error is
			-- Report that configuration parameter is missing
		do
			report_error_message ("No configuration filename has been provided")
		ensure
			has_error: has_error
		end

	report_incorrect_steps (first, last: INTEGER) is
			-- Report that indices of steps, requested to execute, are incorrect
		do
			report_error_message ("Can't start with " + first.out + " and end with " + last.out)
		ensure
			has_error: has_error
		end

	report_incorrect_unfolding_depth (a_depth: INTEGER) is
			-- Report that unfolding depth is incorrect
		do
			report_error_message ("Incorrect unfolding depth: " + a_depth.out + " (should be nonnegative)")
		end

	report_incorrect_once_status_mode (a_mode: STRING) is
			-- Report that `a_mode' is incorrect once status mode
		do
			report_error_message ("Incorrect once status mode: " + a_mode)
		end

	report_incorrect_keep_mode (a_mode: STRING) is
			-- Report that `a_mode' is incorrect original contracts keeping mode
		do
			report_error_message ("Incorrect keep original contracts mode: " + a_mode)
		end

	report_no_classes_to_instrument is
			-- Report that there were no classes to instrument
		do
			report_error_message ("Instrumentation was requested, but there were no classes to instrument")
		ensure
			has_error: has_error
		end

	report_original_system_compilation_failed is
			-- Report that compilation of original system failed
		do
			report_error_message ("Compilation of original system has failed")
		ensure
			has_error: has_error
		end

	report_ppt_not_found (a_name: STRING) is
			-- Report that program point called `a_name' is not found in system
		do
			report_error_message ("Program point " + a_name + " is not found in system " + Settings.project_name)
		ensure
			has_error: has_error
		end

	report_instrumented_system_compilation_failed is
			-- Report that compilation of instrumented system failed
		do
			report_error_message ("Compilation of instrumented system has failed")
		ensure
			has_error: has_error
		end

	report_instrumented_system_execution_failed is
			-- Report that instrumented system hasn't finished successfully
		do
			report_error_message ("Instrumented system execution hasn't finished successfully")
		ensure
			has_error: has_error
		end

	report_daikon_execution_failed is
			-- Report that Daikon execution hasn't finished successfully
		do
			report_error_message ("Daikon execution hasn't finished successfully")
		ensure
			has_error: has_error
		end

	report_print_invariants_execution_failed is
			-- Report that Print Invariants tool execution hasn't finished successfully
		do
			report_error_message ("Print Invariants execution hasn't finished successfully")
		ensure
			has_error: has_error
		end

	report_daikon_dir_not_set (daikon_dir_name: STRING) is
			-- Report that "DAIKONDIR" is not set
		do
			report_error_message ("Environment variable " + daikon_dir_name + " is not set")
		ensure
			has_error: has_error
		end

	report_nonexistent_daikon_dir (daikon_dir_name: STRING; daikon_dir: DIRECTORY_NAME) is
			-- Report that "DAIKONDIR" is set to nonexistent directory
		do
			report_error_message (daikon_dir_name + " is set to non-existent directory " + daikon_dir)
		ensure
			has_error: has_error
		end

	report_java_variables_not_set (jdk_dir_name, jre_dir_name: STRING) is
			-- Report that neither "JDKDIR" not "JREDIR" is set
		do
			report_error_message ("Neither " + jdk_dir_name + " nor " + jre_dir_name + " environment variable is set")
		ensure
			has_error: has_error
		end

	report_nonexistent_java_dir (java_var_name: STRING; java_var: DIRECTORY_NAME) is
			-- Report that variable `java_var_name' is set to nonexistent directory
		do
			report_error_message (java_var_name + " is set to non-existent directory " + java_var)
		ensure
			has_error: has_error
		end

	report_no_java (java_bin: DIRECTORY_NAME) is
			-- Report that JVM is not found
		do
			report_error_message ("Java Virtual Machine is not found in " + java_bin)
		ensure
			has_error: has_error
		end

--	report_no_daikon_bin (daikon_dir: DIRECTORY_NAME) is
--			-- Report that DAIKONBIN can't be set
--		do
--			report_error_message ("Can't find Daikon binaries directory inside " + daikon_dir)
--		ensure
--			has_error: has_error
--		end

	report_configuration_storage_failed is
			-- Report that instrumented system configuration can't be stored
		do
			report_error_message ("Can't store instrumented system to file " + Settings.instrumented_system_name)
		ensure
			has_error: has_error
		end

	report_launch_failed (process_name: STRING) is
			-- Report that launch of process `process_name' failed
		do
			report_error_message ("Can't launch process " + process_name)
		ensure
			has_error: has_error
		end

feature -- Reporting warnings
	report_warning_message (a_message: STRING) is
			-- Report `a_message'
		do
			error_file.put_string ("WARNING: ")
			error_file.put_string (a_message)
			error_file.put_new_line
		end

	report_unknown_cluster (a_cluster_name: STRING) is
			-- Report that cluster calles `a_cluster_name' is unknown
		do
			report_warning_message ("Unknown cluster: " + a_cluster_name)
		end

	report_unknown_class (a_class_name: ET_CLASS_NAME) is
			-- Report that class called `a_class_name' is unknown
		do
			report_warning_message ("Unknown class: " + a_class_name.upper_name)
		end

	report_wrong_class_instrumented (a_class_name: ET_CLASS_NAME) is
			-- Report that class called `a_class_name' can't be instrumented
		do
			report_warning_message ("Sorry, but class " + a_class_name.upper_name + " can't be instrumented at present")
		end

	report_class_not_in_system (a_class_name: ET_CLASS_NAME) is
			-- Report that class called `a_class_name' is not in system and won't be instrumented
		do
			report_warning_message ("Class " + a_class_name.upper_name + " not in system and won't be instrumented")
		end

feature -- Access
	version_number: STRING is
			-- Current version number of CITADEL
		once
			Result := "1.0"
		end

invariant
	version_number_not_empty: not version_number.is_empty
end
