indexing
	description: "Objects that track nonsensical values in Daikon traces"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_NONSENSICAL_TRACKER

inherit
	KL_SHARED_FILE_SYSTEM
		export {NONE}
			all
		end

	CI_SETTINGS_CLIENT

	CI_ERROR_HANDLER_CLIENT

create
	make

feature -- Initialization
	make (a_program_points: like program_points) is
			-- Create tracker, which stores variable information in `a_program_points'
		require
			a_program_points_exists: a_program_points /= Void
		do
			program_points := a_program_points
		ensure
			program_points_set: program_points = a_program_points
		end

feature -- Access
	program_points: HASH_TABLE [CI_PROGRAM_POINT, STRING]
			-- Program points, where variable information is stored

feature -- Basic operations
	read_trace is
			-- Read data trace of instrumented system and find out, what variables were nonsensical
		local
			trace: KI_TEXT_INPUT_FILE
			ppt_name, var_name, var_value: STRING
			ppt: CI_PROGRAM_POINT
			var: CI_VARIABLE
		do
			trace := file_system.new_input_file (Settings.daikon_runtime_input_name)
			trace.open_read
			if not trace.is_open_read then
				error_handler.report_warning_message ("Trace file " + Settings.daikon_runtime_input_name + " can't be read.")
			else
				from
					try_read_line (trace)
				until
					trace.end_of_input
				loop
					ppt_name := trace.last_string.string
					ppt := program_points.item (ppt_name)
					if ppt = Void then
						error_handler.report_warning_message ("Unknown program point: " + ppt_name)
						-- Find next program point
						skip_non_blank (trace)
					else
						-- Read variable values
						from
							try_read_line (trace)
						until
							trace.end_of_input or trace.last_string.is_empty
						loop
							var_name := trace.last_string.string
							try_read_line (trace)
							var_value := trace.last_string.string
							if not trace.end_of_input then
								if var_value.is_equal ({CI_DAIKON_PROGRAM_POINT_PRINTER}.nonsensical_keyword_name) then
									var := ppt.named_variable (var_name)
									if var = Void then
										error_handler.report_warning_message ("Unknown variable: " + var_name)
									else
										var.set_was_nonsensical (True)
									end
								end
							else
								error_handler.report_warning_message ("Trace file " + Settings.daikon_runtime_input_name +
									" has invalid format.")
							end
							try_read_line (trace)
							try_read_line (trace)
						end
					end
					skip_blank (trace)
				end
				trace.close
			end
		end

feature -- Implementation
	try_read_line (a_stream: KI_TEXT_INPUT_STREAM) is
			-- Read line from `a_stream' if it's not end of input
		require
			a_stream_exists: a_stream /= Void
		do
			if not a_stream.end_of_input then
				a_stream.read_line
			end
		end

	skip_non_blank (a_stream: KI_TEXT_INPUT_STREAM) is
			-- Read from `a_stream' until last read line is blank
		require
			a_stream_exists: a_stream /= Void
		do
			from
				try_read_line (a_stream)
			until
				a_stream.end_of_input or a_stream.last_string.is_empty
			loop
				a_stream.read_line
			end
		end

	skip_blank (a_stream: KI_TEXT_INPUT_STREAM) is
			-- Read from `a_stream' until last read line is not blank
		require
			a_stream_exists: a_stream /= Void
		do
			from
				try_read_line (a_stream)
			until
				a_stream.end_of_input or not a_stream.last_string.is_empty
			loop
				a_stream.read_line
			end
		end

invariant
	program_points_exists: program_points /= Void
end
