indexing
	description: "Front end settings and related information%
		% This class is a singleton. To use it inherit from CI_SETTINGS_CLIENT"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_SETTINGS

inherit
	CI_SHARED_TOKEN_CONSTANTS
		redefine
			default_create
		end

	KL_SHARED_FILE_SYSTEM
		redefine
			default_create
		end

	KL_SHARED_STRING_EQUALITY_TESTER
		redefine
			default_create
		end

create {CI_SETTINGS_CLIENT}
	default_create

feature -- Initializtion
	default_create is
			-- Initialize settings with their default values
		local
			class_name_tester: ET_CLASS_NAME_TESTER
		do
			unfolding_depth := {CI_VARIABLE}.default_unfolding_depth
			once_status_mode := once_status_default_mode
			suppression_enabled := True
			keep_original_contracts := keep_original_contracts_default_mode
			create clusters_to_instrument.make_default
			clusters_to_instrument.set_equality_tester (case_insensitive_string_equality_tester)
			create class_name_tester
			create classes_to_instrument.make_default
			classes_to_instrument.set_equality_tester (class_name_tester)
			create extra_changed_classes.make_default
			extra_changed_classes.set_equality_tester (class_name_tester)
		end

feature -- Parameters
	system_name: FILE_NAME
			-- File name, where initial system configuration is stored

	clusters_to_instrument: DS_HASH_SET [STRING]
			-- Names of clusters chosen to instrument

	classes_to_instrument: DS_HASH_SET [ET_CLASS_NAME]
			-- Names of classes that should be instrumented			

feature -- Paths
	project_path: DIRECTORY_NAME is
			-- Name of processed project directory
		require
			system_name_exists: system_name /= Void
		once
			create Result.make_from_string (file_system.dirname (system_name))
		end

	output_path: DIRECTORY_NAME
			-- Directory name, where all output should be placed

	default_output_path: DIRECTORY_NAME is
			-- Default value for `output_path'
		require
			system_name_exists: system_name /= Void
		once
			Result := project_path.twin
			Result.set_subdirectory ("citadel_gens")
		end

	instrumented_system_path: DIRECTORY_NAME is
			-- Directory name, where instrumented system should be placed
		require
			output_path_exists: output_path /= Void
		once
			Result := output_path.twin
			Result.set_subdirectory (project_name + "_instrumented")
		end

	instrumented_cluster_path: DIRECTORY_NAME is
			-- Directory name, where instrumented classes be placed
		require
			output_path_exists: output_path /= Void
		once
			Result := instrumented_system_path.twin
			Result.set_subdirectory ({CI_INSTRUMENTED_SYSTEM}.instrumented_cluster_name)
		end

	instrumented_system_name: FILE_NAME is
			-- File name, where instrumented system configuration is stored
		require
			system_name_exists: system_name /= Void
			output_path_exists: output_path /= Void
		once
			create Result.make
			Result.set_directory (instrumented_system_path)
			Result.set_file_name (project_name)
			Result.add_extension (eiffel_configuration_file_extension)
		end

	instrumented_class_file_name (a_class_name: ET_CLASS_NAME): FILE_NAME is
			-- Name of file where instrumented class called `a_class_name' should be put
		require
			system_name_exists: system_name /= Void
			output_path_exists: output_path /= Void
		do
			create Result.make
			Result.set_directory (instrumented_cluster_path)
			Result.set_file_name (a_class_name.lower_name)
			Result.add_extension ("e")
		ensure
			result_exists: Result /= Void
		end

	daikon_files_path: DIRECTORY_NAME is
			-- Directory name, where daikon input and output should be placed
		require
			output_path_exists: output_path /= Void
		once
			Result := output_path.twin
			Result.set_subdirectory ("daikon_files")
		end

	daikon_static_input_name: FILE_NAME is
			-- Name of Daikon static program point information file
		require
			system_name_exists: system_name /= Void
			output_path_exists: output_path /= Void
		once
			create Result.make
			Result.set_directory (daikon_files_path)
			Result.set_file_name (project_name)
			Result.add_extension (daikon_static_input_extension)
		end

	daikon_runtime_input_name: FILE_NAME is
			-- Name of Daikon runtime program point information file
		require
			system_name_exists: system_name /= Void
			output_path_exists: output_path /= Void
		once
			create Result.make
			Result.set_directory (daikon_files_path)
			Result.set_file_name (project_name)
			Result.add_extension (daikon_runtime_input_extension)
		end

	daikon_output_name: FILE_NAME is
			-- Name of Daikon output file
		require
			system_name_exists: system_name /= Void
			output_path_exists: output_path /= Void
		once
			create Result.make
			Result.set_directory (daikon_files_path)
			Result.set_file_name (project_name)
			Result.add_extension (daikon_output_extension)
			Result.add_extension (archived_extension)
		end

	invariants_name: FILE_NAME is
			-- Name of text file with invariants in Eiffel formal
		require
			system_name_exists: system_name /= Void
			output_path_exists: output_path /= Void
		once
			create Result.make
			Result.set_directory (daikon_files_path)
			Result.set_file_name (project_name)
		end

	annotated_system_path: DIRECTORY_NAME is
			-- Directory name, where annotated source code should be placed
		require
			output_path_exists: output_path /= Void
		once
			Result := output_path.twin
			Result.set_subdirectory (project_name + "_annotated")
		end

	annotated_class_file_name (a_class_name: ET_CLASS_NAME): FILE_NAME is
			-- Name of file where annotated class called `a_class_name' should be put
		require
			system_name_exists: system_name /= Void
			output_path_exists: output_path /= Void
		do
			create Result.make
			Result.set_directory (annotated_system_path)
			Result.set_file_name (a_class_name.lower_name)
			Result.add_extension ("e")
		ensure
			result_exists: Result /= Void
		end

	original_universe_file_name: FILE_NAME is
			-- Name of file, where original universe should be stored
		require
			output_path_exists: output_path /= Void
		once
			create Result.make
			Result.set_directory (output_path)
			Result.set_file_name ("universe")
		ensure
			result_exists: Result /= Void
		end

feature -- Extensions
	eiffel_configuration_file_extension: STRING is "ecf"
			-- Extension of Eiffel configuration files

	daikon_static_input_extension: STRING is "decls"
			-- Extension of Daikon static program point information files

	daikon_runtime_input_extension: STRING is "dtrace"
			-- Extension of Daikon runtime program point information files

	daikon_output_extension: STRING is "inv"
			-- Extension of Daikon invariants file

	exe_extension: STRING is
			-- Extension of executable files (without preceding dot)
		local
			ext: STRING
		once
			ext := file_system.exe_extension
			if not ext.is_empty then
				Result := ext.substring (2, ext.count)
			else
				Result := ""
			end
		end

	archived_extension: STRING is "gz"
			-- Extension that is added by currently used archiver

feature -- Options
	system_target: STRING
			-- Target name for initial system

	instrumented_system_arguments: STRING
			-- Arguments to pass to instrumented system execution

	daikon_arguments: STRING
			-- Arguments to pass to Daikon

	unfolding_depth: INTEGER
			-- Depth to which ufolding is done

	unfold_printable_variables: BOOLEAN
			-- Are printable variables unfolded?

	use_functions: BOOLEAN
			-- Do zero-argument functions take part in unfolding?

	use_any_queries: BOOLEAN
			-- Do queries implemented in `ANY' take part in unfolding?

	use_obsolete_queries: BOOLEAN
			-- Do obsolete queries take part in unfolding?

	use_inapplicable_queries: BOOLEAN
			-- Do queries from feature clause marked "Inapplicable" take part in unfolding?			

	none_mode: INTEGER is 0
			-- "None" value for `once_status_mode' option

	incomplete_mode: INTEGER is 1
			-- "Incomplete" value for `once_status_mode' option

	slow_mode: INTEGER is 2
			-- "Slow" value for `once_status_mode' option			

	unsafe_mode: INTEGER is 3
			-- "Unsafe" value for `once_status_mode' option						

	once_status_default_mode: INTEGER is 1
			-- Default value of `once_status_mode' option

	once_status_mode: INTEGER
			-- Mode of recording and cheching status of once fucntions during instrumented system execution for using them in unfolding:
			-- "None": once functions are not used in unfolding
			-- "Incomplete": only once functions from instrumented classes are used in unfolding
			-- "Slow": all once functions are used, but requires modification of all system, not just instrumented classes
			-- "Unsafe": once functions are used without watching their status (can lead to system behavior alteration)

	workbench_mode: BOOLEAN
			-- Is instrumented system compiled in workbench mode?
			-- (False means it is compiled in finalized mode)

	suppression_enabled: BOOLEAN
			-- If true, assertions are not printed in precondition and postcondition of exported routines, if they present in invariant

	instrumentation_enabled: BOOLEAN
			-- Should instrumentation phase be done?

	compilation_enabled: BOOLEAN
			-- Should instrumented system be compiled?

	execution_enabled: BOOLEAN
			-- Should instrumented system be executed?

	inference_enabled: BOOLEAN
			-- Should assertion inference be done?

	annotation_enabled: BOOLEAN
			-- Should code be annotated with inferred assertions?

	remove_mode: INTEGER is 0
			-- "Remove" value for `keep_original_contracts' option

	keep_mode: INTEGER is 1
			-- "Keep" value for `keep_original_contracts' option

	flatten_mode: INTEGER is 2
			-- "Flatten" value for `keep_original_contracts' option

	keep_original_contracts_default_mode: INTEGER is 2
			-- Default value of `keep_original_contracts' option			

	keep_original_contracts: INTEGER
			-- Should original contracts be kept in annotated system?

feature -- Setting
	set_system_name (a_name: STRING) is
			-- Set `system_name' to `a_name'
		require
			a_name_not_empty: a_name /= Void and then not a_name.is_empty
		once
			create system_name.make_from_string (file_system.canonical_pathname (file_system.absolute_pathname (a_name)))
			if output_path = Void then
				output_path := default_output_path
			end
		ensure
			system_name_exists: system_name /= Void
		end

	set_system_target (a_target: like system_target) is
			-- Set `system_target' to `a_target'
		require
			a_target_not_empty: a_target /= Void and then not a_target.is_empty
		once
			system_target := a_target
		ensure
			system_target_set: system_target = a_target
		end

	set_output_path (a_path: STRING) is
			-- Set `output_path' to `a_path'
		require
			a_path_not_empty: a_path /= Void and then not a_path.is_empty
		once
			create output_path.make_from_string (file_system.canonical_pathname (file_system.absolute_pathname (a_path)))
		ensure
			output_path_exists: output_path /= Void
		end

	set_classes_to_instrument (a_classes: like classes_to_instrument) is
			-- Set `classes_to_instrument' to `a_classes'
		require
			a_classes_exists: a_classes /= Void
		once
			classes_to_instrument := a_classes
		ensure
			classes_to_instrument_set: classes_to_instrument = a_classes
		end

	set_unfolding_depth (a_depth: INTEGER) is
			-- Set `unfolding_depth' to `a_depth'
		require
			a_depth_non_negative: a_depth >= 0
		once
			unfolding_depth := a_depth
		ensure
			unfolding_depth_set: unfolding_depth = a_depth
		end

	set_unfold_printable_variables (b: BOOLEAN) is
			-- Set `unfold_printable_variables' to `b'
		once
			unfold_printable_variables := b
		ensure
			unfold_printable_variables_set: unfold_printable_variables = b
		end

	set_use_functions (b: BOOLEAN) is
			-- Set `use_functions' to `b'
		once
			use_functions := b
		ensure
			use_functions_set: use_functions = b
		end

	set_use_any_queries (b: BOOLEAN) is
			-- Set `use_any_queries' to `b'
		once
			use_any_queries := b
		ensure
			use_any_queries_set: use_any_queries = b
		end

	set_use_obsolete_queries (b: BOOLEAN) is
			-- Set `use_obsolete_queries' to `b'
		once
			use_obsolete_queries := b
		ensure
			use_obsolete_queries_set: use_obsolete_queries = b
		end

	set_use_inapplicable_queries (b: BOOLEAN) is
			-- Set `use_inapplicable_queries' to `b'
		once
			use_inapplicable_queries := b
		ensure
			use_inapplicable_queries_set: use_inapplicable_queries = b
		end

	set_once_status_mode (a_mode: INTEGER) is
			-- Set `once_status_mode' to `a_mode'
		once
			once_status_mode := a_mode
		ensure
			once_status_mode_set: once_status_mode = a_mode
		end

	set_workbench_mode (b: BOOLEAN) is
			-- Set `workbench_mode' to `b'
		once
			workbench_mode := b
		ensure
			workbench_mode_set: workbench_mode = b
		end

	set_instrumented_system_arguments (an_args: STRING) is
			-- Set `instrumented_system_arguments' to `an_args'
		once
			instrumented_system_arguments := an_args
		ensure
			instrumented_system_arguments_set: instrumented_system_arguments = an_args
		end

	set_daikon_arguments (an_args: STRING) is
			-- Set `daikon_arguments' to `an_args'
		once
			daikon_arguments := an_args
		ensure
			daikon_arguments_set: daikon_arguments = an_args
		end

	set_suppression_enabled (b: BOOLEAN) is
			-- Set `suppression_enabled' to `b'
		once
			suppression_enabled := b
		ensure
			suppression_enabled_set: suppression_enabled = b
		end

	set_instrumentation_enabled (b: BOOLEAN) is
			-- Set `instrumentation_enabled' to `b'
		once
			instrumentation_enabled := b
		ensure
			instrumentation_enabled_set: instrumentation_enabled = b
		end

	set_compilation_enabled (b: BOOLEAN) is
			-- Set `compilation_enabled' to `b'
		once
			compilation_enabled := b
		ensure
			compilation_enabled_set: compilation_enabled = b
		end

	set_execution_enabled (b: BOOLEAN) is
			-- Set `execution_enabled' to `b'
		once
			execution_enabled := b
		ensure
			execution_enabled_set: execution_enabled = b
		end

	set_inference_enabled (b: BOOLEAN) is
			-- Set `inference_enabled' to `b'
		once
			inference_enabled := b
		ensure
			inference_enabled_set: inference_enabled = b
		end

	set_annotation_enabled (b: BOOLEAN) is
			-- Set `annotation_enabled' to `b'
		once
			annotation_enabled := b
		ensure
			annotation_enabled_set: annotation_enabled = b
		end

	set_keep_original_contracts_mode (a_mode: INTEGER) is
			-- Set `keep_original_contracts' to `a_mode'
		do
			keep_original_contracts := a_mode
		ensure
			keep_original_contracts = a_mode
		end

feature -- Access
	project_name: STRING is
			-- Processed project name
		require
			system_name_exists: system_name /= Void
		local
			ecf_name: STRING
		once
			ecf_name := file_system.basename (system_name)
			check ecf_name.count > eiffel_configuration_file_extension.count + 1 end
			Result := ecf_name.substring (1, ecf_name.count - eiffel_configuration_file_extension.count - 1)
		end

	subcluster_separator: CHARACTER is '/'
			-- Separator between cluster and subcluster names

	instrumented_cluster: ET_CLUSTER is
			-- Cluster for instrumented classes
		once
			create {ET_ECF_CLUSTER} Result.make ({CI_INSTRUMENTED_SYSTEM}.instrumented_cluster_name, instrumented_cluster_path)
		end

	extra_changed_classes: DS_HASH_SET [ET_CLASS_NAME]

feature -- Status report
	is_unfoldable (a_variable: CI_VARIABLE): BOOLEAN is
			-- Is `a_variable' unfoldable with current settings?
		require
			a_variable_exists: a_variable /= Void
		do
			Result := unfold_printable_variables or not a_variable.type.kind.is_printable
		end

	is_instrumentable (a_feature: ET_FEATURE): BOOLEAN is
			-- Will `a_feature' be instrumented?
		local
			r: ET_INTERNAL_ROUTINE
		do
			-- Yes if query is an internal function and its implementation class is instrumented
			r ?= a_feature
			Result := r /= Void and then classes_to_instrument.has (r.implementation_class.name)
		ensure
			only_routines: Result implies (a_feature.is_function or a_feature.is_procedure)
			if_impl_class_is_instrumented: Result implies classes_to_instrument.has (a_feature.implementation_class.name)
		end

	is_valid_query_variable (a_query: ET_QUERY): BOOLEAN is
			-- Can a query variable be based on `a_query'
		require
			a_query_exists: a_query /= Void
		do
			if use_functions then
				Result := a_query.arguments_count = 0	-- zero-argument query
			else
				Result := a_query.is_attribute			-- only attribute
			end
			inspect once_status_mode
			when none_mode then
				Result := Result and not a_query.is_once	-- Not once
			when incomplete_mode then
				Result := Result and (not a_query.is_once or classes_to_instrument.has (a_query.implementation_class.name))
					-- Not once from uninstrumented class
			else
			end
			Result := Result and
				(use_any_queries or not a_query.implementation_class.name.same_class_name (tokens.any_class_name)) and	-- not from ANY
				(use_obsolete_queries or a_query.obsolete_message = Void) and							-- not obsolete			
				(use_inapplicable_queries or not a_query.feature_clause.header_break.text.has_substring (tokens.inapplicable_name)) -- not inapplicable
		end

	is_watched_once (a_query: ET_QUERY): BOOLEAN is
			-- Should we check that `a_query' has already been called before printing it in unfolding?
		require
			a_query_exists: a_query /= Void
		do
			Result := a_query.is_once and (once_status_mode = incomplete_mode or once_status_mode = slow_mode)
		end

	is_any_changed: BOOLEAN is
			-- Is class ANY changed in instrumented system
		do
			Result := classes_to_instrument.has (tokens.any_class_name) or once_status_mode = slow_mode
		end

	is_uninteresting (an_assertion: CI_INFERRED_ASSERTION): BOOLEAN is
			-- Is `an_assertions' one of assertions considered useless?
		do
			Result := an_assertion.is_current_exists or an_assertion.is_constant_address or an_assertion.is_current_not_changed
		end

invariant
	eiffel_configuration_file_extension_valid: system_name.is_extension_valid (eiffel_configuration_file_extension)
	daikon_static_input_extension_valid: daikon_static_input_name.is_extension_valid (daikon_static_input_extension)
	daikon_runtime_input_extension_valid: daikon_runtime_input_name.is_extension_valid (daikon_runtime_input_extension)
	clusters_to_instrument_exists: clusters_to_instrument /= Void
	classes_to_instrument_exists: classes_to_instrument /= Void
end
