indexing
	description: "AST pretty printers.%
	% Class fixes some parent bugs and make code syntactically correct. When bugs will be fixed in parent, class should be removed.%
	% Bugs fixed: absence of breaks after feature declarations; duplication of infix and prefix feature names;%
	% absence of space between 'require' and 'else' in preconditions; printing 'obsolete' twice in obsolete messages;%
	% class second indexing (ISE bug?); current, feature and expression addresses (nonstandart); comment printing"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_AST_PRETTY_PRINTER

inherit
	ET_AST_PRETTY_PRINTER
		redefine
			process_attribute,
			process_class,
			process_constant_attribute,
			process_current_address,
			process_deferred_function,
			process_deferred_procedure,
			process_do_procedure,
			process_do_function,
			process_expression_address,
			process_external_function,
			process_external_procedure,
			process_feature_address,
			process_feature_clause,
			process_once_function,
			process_once_procedure,
			process_preconditions,
			process_unique_attribute,
			print_comment
		end

create
	make, make_null

feature -- Processing
	process_attribute (a_feature: ET_ATTRIBUTE) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_synonym: ET_FEATURE
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_assigner: ET_ASSIGNER
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_declared_type: ET_DECLARED_TYPE
			l_type: ET_TYPE
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
				-- The AST may or may not contain the colon.
				-- So we have to print it explicitly here.
			l_declared_type := a_feature.declared_type
			l_type := l_declared_type.type
			tokens.colon_symbol.process (Current)
			comment_finder.add_excluded_node (l_type)
			comment_finder.find_comments (l_declared_type, comment_list)
			comment_finder.reset_excluded_nodes
			print_space
			l_type.process (Current)
			l_assigner := a_feature.assigner
			if l_assigner /= Void then
				print_space
				l_assigner.process (Current)
			end
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
				-- Print header comment.
			indent
			process_comments
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_class (a_class: ET_CLASS) is
			-- Process `a_class'.
		local
			an_indexing: ET_INDEXING_LIST
			a_frozen: ET_KEYWORD
			a_class_mark: ET_KEYWORD
			an_external: ET_KEYWORD
			a_formal_parameters: ET_FORMAL_PARAMETER_LIST
			l_obsolete_message: ET_OBSOLETE
			l_obsolete_string: ET_MANIFEST_STRING
			a_parents: ET_PARENT_LIST
			a_creators: ET_CREATOR_LIST
			a_convert_features: ET_CONVERT_FEATURE_LIST
			an_invariants: ET_INVARIANTS
		do
			process_break (a_class.leading_break)
			process_comments
-- TODO: add new-line if a comment has been printed.
			an_indexing := a_class.first_indexing
			if an_indexing /= Void then
				an_indexing.process (Current)
				process_comments
				print_new_line
				print_new_line
			end
			a_frozen := a_class.frozen_keyword
			if a_frozen /= Void then
				a_frozen.process (Current)
				print_space
			end
			a_class_mark := a_class.class_mark
			if a_class_mark /= Void then
				a_class_mark.process (Current)
				print_space
			end
			an_external := a_class.external_keyword
			if an_external /= Void then
				an_external.process (Current)
				print_space
			end
			a_class.class_keyword.process (Current)
			print_space
			a_class.name.process (Current)
			a_formal_parameters := a_class.formal_parameters
			if a_formal_parameters /= Void then
				if a_formal_parameters.is_empty then
						-- Do not print empty brackets, but keep the comments if any.
					comment_finder.find_comments (a_formal_parameters, comment_list)
				else
					print_space
					a_formal_parameters.process (Current)
				end
			end
			process_comments
			print_new_line
			print_new_line
			l_obsolete_message := a_class.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				dedent
				process_comments
				print_new_line
				print_new_line
			end
			a_parents := a_class.parents
			if a_parents /= Void then
				a_parents.process (Current)
				process_comments
				print_new_line
				print_new_line
			end
			a_creators := a_class.creators
			if a_creators /= Void then
				a_creators.process (Current)
				process_comments
				print_new_line
				print_new_line
			end
			a_convert_features := a_class.convert_features
			if a_convert_features /= Void then
				a_convert_features.process (Current)
				process_comments
				print_new_line
				print_new_line
			end
			process_features (a_class)
			an_invariants := a_class.invariants
			if an_invariants /= Void then
				an_invariants.process (Current)
				process_comments
				print_new_line
				print_new_line
			end
			-- Second indexing is not printed, because it causes syntaxt error, if after attribute
			a_class.end_keyword.process (Current)
-- TODO: print the comment on the same line as the end keyword.
			process_comments
		end

	process_constant_attribute (a_feature: ET_CONSTANT_ATTRIBUTE) is
			-- Process `a_feature'.		
		local
			l_frozen_keyword: ET_TOKEN
			l_synonym: ET_FEATURE
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_assigner: ET_ASSIGNER
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_declared_type: ET_DECLARED_TYPE
			l_type: ET_TYPE
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
				-- The AST may or may not contain the colon.
				-- So we have to print it explicitly here.
			l_declared_type := a_feature.declared_type
			l_type := l_declared_type.type
			tokens.colon_symbol.process (Current)
			comment_finder.add_excluded_node (l_type)
			comment_finder.find_comments (l_declared_type, comment_list)
			comment_finder.reset_excluded_nodes
			print_space
			l_type.process (Current)
			print_space
			l_assigner := a_feature.assigner
			if l_assigner /= Void then
				l_assigner.process (Current)
				print_space
			end
			tokens.is_keyword.process (Current)
			process_break (a_feature.is_keyword.break)
			print_space
			a_feature.constant.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
				-- Print header comment.
			indent
			process_comments
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_current_address (an_expression: ET_CURRENT_ADDRESS) is
			-- Process `an_expression'.
		do
			tokens.dollar_symbol.process (Current)
			tokens.current_keyword.process (Current)
			comment_finder.find_comments (an_expression, comment_list)
		end

	process_deferred_function (a_feature: ET_DEFERRED_FUNCTION) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_declared_type: ET_DECLARED_TYPE
			l_type: ET_TYPE
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_assigner: ET_ASSIGNER
			l_obsolete_string: ET_MANIFEST_STRING
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
				-- The AST may or may not contain the colon.
				-- So we have to print it explicitly here.
			l_declared_type := a_feature.declared_type
			l_type := l_declared_type.type
			tokens.colon_symbol.process (Current)
			comment_finder.add_excluded_node (l_type)
			comment_finder.find_comments (l_declared_type, comment_list)
			comment_finder.reset_excluded_nodes
			print_space
			l_type.process (Current)
			l_assigner := a_feature.assigner
			if l_assigner /= Void then
				print_space
				l_assigner.process (Current)
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			a_feature.deferred_keyword.process (Current)
			process_comments
			print_new_line
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_deferred_procedure (a_feature: ET_DEFERRED_PROCEDURE) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_obsolete_string: ET_MANIFEST_STRING
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			a_feature.deferred_keyword.process (Current)
			process_comments
			print_new_line
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_do_function (a_feature: ET_DO_FUNCTION) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_declared_type: ET_DECLARED_TYPE
			l_type: ET_TYPE
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_assigner: ET_ASSIGNER
			l_obsolete_string: ET_MANIFEST_STRING
			l_locals: ET_LOCAL_VARIABLE_LIST
			l_compound: ET_COMPOUND
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
				-- The AST may or may not contain the colon.
				-- So we have to print it explicitly here.
			l_declared_type := a_feature.declared_type
			l_type := l_declared_type.type
			tokens.colon_symbol.process (Current)
			comment_finder.add_excluded_node (l_type)
			comment_finder.find_comments (l_declared_type, comment_list)
			comment_finder.reset_excluded_nodes
			print_space
			l_type.process (Current)
			l_assigner := a_feature.assigner
			if l_assigner /= Void then
				print_space
				l_assigner.process (Current)
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			l_locals := a_feature.locals
			if l_locals /= Void then
				l_locals.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.compound
			if l_compound /= Void then
				l_compound.process (Current)
			else
				tokens.do_keyword.process (Current)
			end
			process_comments
			print_new_line
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.rescue_clause
			if l_compound /= Void then
				l_compound.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_do_procedure (a_feature: ET_DO_PROCEDURE) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_obsolete_string: ET_MANIFEST_STRING
			l_locals: ET_LOCAL_VARIABLE_LIST
			l_compound: ET_COMPOUND
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			l_locals := a_feature.locals
			if l_locals /= Void then
				l_locals.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.compound
			if l_compound /= Void then
				l_compound.process (Current)
			else
				tokens.do_keyword.process (Current)
			end
			process_comments
			print_new_line
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.rescue_clause
			if l_compound /= Void then
				l_compound.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_expression_address (an_expression: ET_EXPRESSION_ADDRESS) is
			-- Process `an_expression'.
		do
			tokens.dollar_symbol.process (Current)
			an_expression.expression.process (Current)
			comment_finder.find_comments (an_expression, comment_list)
		end

	process_external_function (a_feature: ET_EXTERNAL_FUNCTION) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_declared_type: ET_DECLARED_TYPE
			l_type: ET_TYPE
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_assigner: ET_ASSIGNER
			l_obsolete_string: ET_MANIFEST_STRING
			l_external_language: ET_EXTERNAL_LANGUAGE
			l_external_alias: ET_EXTERNAL_ALIAS
			l_manifest_string: ET_MANIFEST_STRING
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
				-- The AST may or may not contain the colon.
				-- So we have to print it explicitly here.
			l_declared_type := a_feature.declared_type
			l_type := l_declared_type.type
			tokens.colon_symbol.process (Current)
			comment_finder.add_excluded_node (l_type)
			comment_finder.find_comments (l_declared_type, comment_list)
			comment_finder.reset_excluded_nodes
			print_space
			l_type.process (Current)
			l_assigner := a_feature.assigner
			if l_assigner /= Void then
				print_space
				l_assigner.process (Current)
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				dedent
				process_comments
				print_new_line
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			tokens.external_keyword.process (Current)
			l_external_language := a_feature.language
			l_manifest_string := l_external_language.manifest_string
			comment_finder.add_excluded_node (l_manifest_string)
			comment_finder.find_comments (l_external_language, comment_list)
			comment_finder.reset_excluded_nodes
			indent
			process_comments
			print_new_line
			l_manifest_string.process (Current)
			process_comments
			print_new_line
			dedent
			l_external_alias := a_feature.alias_clause
			if l_external_alias /= Void then
				tokens.alias_keyword.process (Current)
				l_manifest_string := l_external_alias.manifest_string
				comment_finder.add_excluded_node (l_manifest_string)
				comment_finder.find_comments (l_external_alias, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_manifest_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_external_procedure (a_feature: ET_EXTERNAL_PROCEDURE) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_obsolete_string: ET_MANIFEST_STRING
			l_external_language: ET_EXTERNAL_LANGUAGE
			l_external_alias: ET_EXTERNAL_ALIAS
			l_manifest_string: ET_MANIFEST_STRING
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				dedent
				process_comments
				print_new_line
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			tokens.external_keyword.process (Current)
			l_external_language := a_feature.language
			l_manifest_string := l_external_language.manifest_string
			comment_finder.add_excluded_node (l_manifest_string)
			comment_finder.find_comments (l_external_language, comment_list)
			comment_finder.reset_excluded_nodes
			indent
			process_comments
			print_new_line
			l_manifest_string.process (Current)
			process_comments
			print_new_line
			dedent
			l_external_alias := a_feature.alias_clause
			if l_external_alias /= Void then
				tokens.alias_keyword.process (Current)
				l_manifest_string := l_external_alias.manifest_string
				comment_finder.add_excluded_node (l_manifest_string)
				comment_finder.find_comments (l_external_alias, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_manifest_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_feature_address (an_expression: ET_FEATURE_ADDRESS) is
			-- Process `an_expression'.
		do
			tokens.dollar_symbol.process (Current)
			an_expression.name.process (Current)
			comment_finder.find_comments (an_expression, comment_list)
		end

	process_feature_clause (a_feature_clause: ET_FEATURE_CLAUSE) is
			-- Process `a_feature_clause'.
		do
			Precursor (a_feature_clause)
			if not comment_printed then
				print_new_line
			end
		end

	process_once_function (a_feature: ET_ONCE_FUNCTION) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_declared_type: ET_DECLARED_TYPE
			l_type: ET_TYPE
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_assigner: ET_ASSIGNER
			l_obsolete_string: ET_MANIFEST_STRING
			l_locals: ET_LOCAL_VARIABLE_LIST
			l_compound: ET_COMPOUND
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
				-- The AST may or may not contain the colon.
				-- So we have to print it explicitly here.
			l_declared_type := a_feature.declared_type
			l_type := l_declared_type.type
			tokens.colon_symbol.process (Current)
			comment_finder.add_excluded_node (l_type)
			comment_finder.find_comments (l_declared_type, comment_list)
			comment_finder.reset_excluded_nodes
			print_space
			l_type.process (Current)
			l_assigner := a_feature.assigner
			if l_assigner /= Void then
				print_space
				l_assigner.process (Current)
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			l_locals := a_feature.locals
			if l_locals /= Void then
				l_locals.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.compound
			if l_compound /= Void then
				l_compound.process (Current)
			else
				tokens.once_keyword.process (Current)
			end
			process_comments
			print_new_line
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.rescue_clause
			if l_compound /= Void then
				l_compound.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_once_procedure (a_feature: ET_ONCE_PROCEDURE) is
			-- Process `a_feature'.
		local
			l_frozen_keyword: ET_TOKEN
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_synonym: ET_FEATURE
			l_arguments: ET_FORMAL_ARGUMENT_LIST
			l_indexing: ET_INDEXING_LIST
			l_obsolete_message: ET_OBSOLETE
			l_preconditions: ET_PRECONDITIONS
			l_postconditions: ET_POSTCONDITIONS
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_is_keyword: ET_KEYWORD
			l_obsolete_string: ET_MANIFEST_STRING
			l_locals: ET_LOCAL_VARIABLE_LIST
			l_compound: ET_COMPOUND
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				if l_arguments.is_empty then
						-- Do not print empty parentheses, but keep the comments if any.
					comment_finder.find_comments (l_arguments, comment_list)
				else
					print_space
					l_arguments.process (Current)
				end
			end
			print_space
			tokens.is_keyword.process (Current)
			l_is_keyword := a_feature.is_keyword
			if l_is_keyword /= Void then
				process_break (l_is_keyword.break)
			end
			indent
			process_comments
			print_new_line
			l_indexing := a_feature.first_indexing
			if l_indexing /= Void then
				l_indexing.process (Current)
				process_comments
				print_new_line
			end
			l_obsolete_message := a_feature.obsolete_message
			if l_obsolete_message /= Void then
				tokens.obsolete_keyword.process (Current)
				l_obsolete_string := l_obsolete_message.manifest_string
				comment_finder.add_excluded_node (l_obsolete_string)
				comment_finder.find_comments (l_obsolete_message, comment_list)
				comment_finder.reset_excluded_nodes
				indent
				process_comments
				print_new_line
				l_obsolete_string.process (Current)
				process_comments
				print_new_line
				dedent
			end
			l_preconditions := a_feature.preconditions
			if l_preconditions /= Void then
				l_preconditions.process (Current)
				process_comments
				print_new_line
			end
			l_locals := a_feature.locals
			if l_locals /= Void then
				l_locals.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.compound
			if l_compound /= Void then
				l_compound.process (Current)
			else
				tokens.once_keyword.process (Current)
			end
			process_comments
			print_new_line
			l_postconditions := a_feature.postconditions
			if l_postconditions /= Void then
				l_postconditions.process (Current)
				process_comments
				print_new_line
			end
			l_compound := a_feature.rescue_clause
			if l_compound /= Void then
				l_compound.process (Current)
				process_comments
				print_new_line
			end
			a_feature.end_keyword.process (Current)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
			dedent

			dedent
			print_new_line
			print_new_line
		end

	process_preconditions (a_list: ET_PRECONDITIONS) is
			-- Process `a_list'.
		local
			an_else_keyword: ET_TOKEN
		do
			if a_list.is_empty then
					-- Do not print empty preconditions, but keep the comments if any.
				comment_finder.find_comments (a_list, comment_list)
			else
				a_list.require_keyword.process (Current)
				an_else_keyword := a_list.else_keyword
				if an_else_keyword /= Void then
					print_space
					an_else_keyword.process (Current)
				end
				print_new_line
				indent
				process_comments
				process_assertions (a_list)
				process_comments
				dedent
			end
		end

	process_unique_attribute (a_feature: ET_UNIQUE_ATTRIBUTE) is
			-- Process `a_feature'
		local
			l_frozen_keyword: ET_TOKEN
			l_synonym: ET_FEATURE
			l_semicolon: ET_SEMICOLON_SYMBOL
			l_assigner: ET_ASSIGNER
			l_extended_feature_name: ET_EXTENDED_FEATURE_NAME
			l_feature_name: ET_FEATURE_NAME
			l_alias_name: ET_ALIAS_NAME
			l_declared_type: ET_DECLARED_TYPE
			l_type: ET_TYPE
		do
			indent

--			Precursor (a_feature)
			from
				l_synonym := a_feature
			until
				l_synonym = Void
			loop
				l_frozen_keyword := l_synonym.frozen_keyword
				if l_frozen_keyword /= Void then
					l_frozen_keyword.process (Current)
					print_space
				end
				l_extended_feature_name := l_synonym.extended_name
				l_feature_name := l_extended_feature_name.feature_name
				l_alias_name := l_extended_feature_name.alias_name
				l_synonym := l_synonym.synonym
				l_feature_name.process (Current)
				if l_alias_name /= Void and not l_feature_name.is_alias then
					print_space
					l_alias_name.process (Current)
					comment_finder.add_excluded_node (l_alias_name)
				end
				comment_finder.add_excluded_node (l_feature_name)
				comment_finder.find_comments (l_extended_feature_name, comment_list)
				comment_finder.reset_excluded_nodes
				if l_synonym /= Void then
						-- The AST may or may not contain the comma.
						-- So we have to print it explicitly here.
					tokens.comma_symbol.process (Current)
					print_space
				end
			end
				-- The AST may or may not contain the colon.
				-- So we have to print it explicitly here.
			l_declared_type := a_feature.declared_type
			l_type := l_declared_type.type
			tokens.colon_symbol.process (Current)
			comment_finder.add_excluded_node (l_type)
			comment_finder.find_comments (l_declared_type, comment_list)
			comment_finder.reset_excluded_nodes
			print_space
			l_type.process (Current)
			print_space
			l_assigner := a_feature.assigner
			if l_assigner /= Void then
				l_assigner.process (Current)
				print_space
			end
			tokens.is_keyword.process (Current)
			process_break (a_feature.is_keyword.break)
			print_space
			tokens.unique_keyword.process (Current)
			process_break (a_feature.unique_keyword.break)
			l_semicolon := a_feature.semicolon
			if l_semicolon /= Void then
					-- Do not print the semicolon, but keep track of its comments if any.
				process_break (l_semicolon.break)
			end
				-- Print header comment.
			indent
			process_comments
			dedent

			dedent
			print_new_line
			print_new_line
		end

feature {NONE} -- Comments
	print_comment (a_break: ET_BREAK) is
			-- Print comment held in `a_break', followed by a new-line.
		require else
			a_break_exists: a_break /= Void
		do
			comment_printed := False
			if not indentation_printed then
				print_indentation
			end

			print_string (trimmed (a_break.text))

			print_new_line
			comment_printed := True
		end

	trimmed (a_string: STRING): STRING is
			-- `a_string' without leading and trailing breaks
		local
			start_pos, end_pos: INTEGER
		do
			from
				start_pos := 1
			until
				not a_string.item (start_pos).is_space
			loop
				start_pos := start_pos + 1
			end
			from
				end_pos := a_string.count
			until
				not a_string.item (end_pos).is_space
			loop
				end_pos := end_pos - 1
			end
			Result := a_string.substring (start_pos, end_pos)
		end

end
