indexing
	description: "Stacks of nested closures. %
		% Useful as a context for identifiers inside inline agents."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_CLOSURE_STACK

inherit
	ET_CLOSURE
		undefine
			copy,
			is_equal
		redefine
			type,
			arguments,
			locals
		end

	LINKED_STACK [ET_CLOSURE]
		redefine
			put,
			remove
		end

create
	make

feature -- Access
	type: ET_TYPE is
			-- Return type;
			-- Void for procedures
		do
			if not is_empty then
				Result := item.type
			end
		end

	arguments: ET_FORMAL_ARGUMENT_LIST
			-- Formal arguments: union of formal arguments of all closures from `closures'
			-- Void if no arguments

	locals: ET_LOCAL_VARIABLE_LIST
			-- Local variables: union of locals of all closures from `closures'
			-- Void if no local variables

feature -- Element change
	put (a_closure: ET_CLOSURE) is
			-- Put `a_closure' on top
		local
			i: INTEGER
		do
			Precursor (a_closure)

			if a_closure /= Void and then a_closure.arguments /= Void then
				if arguments = Void then
					create arguments.make_with_capacity (a_closure.arguments.count)
				end
				from
					i := a_closure.arguments.count
				until
					i < 1
				loop
					arguments.force_first (a_closure.arguments.item (i))
					i := i - 1
				end
			end
			if a_closure /= Void and a_closure.locals /= Void then
				if locals = Void then
					create locals.make_with_capacity (a_closure.locals.count)
				end
				from
					i := a_closure.locals.count
				until
					i < 1
				loop
					locals.force_first (a_closure.locals.item (i))
					i := i - 1
				end
			end
		end

	remove is
			-- Remove closure from top
		local
			closure: ET_CLOSURE
			i: INTEGER
		do
			closure := item
			Precursor

			if closure /= Void and then closure.arguments /= Void then
				from
					i := closure.arguments.count
				until
					i < 1
				loop
					arguments.remove_first
					i := i - 1
				end
				if arguments.count = 0 then
					arguments := Void
				end
			end
			if closure /= Void and then closure.locals /= Void then
				from
					i := closure.locals.count
				until
					i < 1
				loop
					locals.remove_first
					i := i - 1
				end
				if locals.count = 0 then
					locals := Void
				end
			end
		end
end
