indexing
	description: "Objects that rename fetures in AST nodes from ancestor when these nodes should be used in descendant context"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_FEATURE_RENAMER

inherit
	CI_AST_MODIFIER
		rename
			make as make_with_universe,
			context as ancestor
		export {NONE}
			make_with_universe
		redefine
			ancestor,
			process_bang_instruction,
			process_bracket_expression,
			process_call_agent,
			process_call_expression,
			process_call_instruction,
			process_create_expression,
			process_create_instruction,
			process_identifier,
			process_infix_expression,
			process_prefix_expression,
			process_static_call_expression,
			process_static_call_instruction
		end

create
	make

feature -- Initialization
	make (an_ancestor_feature: like ancestor_feature; a_descendant_feature: like descendant_feature;
		an_ancestor: like ancestor; a_descendant: like descendant; a_universe: like universe) is
			-- Create a renamer inside `an_ancestor_feature' (may be Void for class-wide renaming
			-- or if should be determined automatically) from `an_ancestor'
			-- to `a_descendant_feature' (may be Void for class-wide renaming) from `a_descendant' within `a_universe'
		require
			an_ancestor_exists: an_ancestor /= Void
			a_descendant_exists: a_descendant /= Void
			a_universe_exists: a_universe /= Void
		local
			i: INTEGER
		do
			make_with_universe (a_universe)
			ancestor := an_ancestor
			descendant := a_descendant
			descendant_feature := a_descendant_feature
			ancestor_feature := an_ancestor_feature
			if ancestor_feature = Void and descendant_feature /= Void then
				ancestor_feature := seeded_feature (ancestor, descendant_feature.first_seed)
				if ancestor_feature = Void and descendant_feature.other_seeds /= Void then
					from
						i := 1
					until
						i > descendant_feature.other_seeds.count or ancestor_feature /= Void
					loop
						ancestor_feature := seeded_feature (ancestor, descendant_feature.other_seeds.item (i))
						i := i + 1
					end
				end
			end
			create ancestor_type_evaluator.make (ancestor_feature, ancestor, universe)
			create descendant_type_evaluator.make (descendant_feature, descendant, universe)
		ensure
			ancestor_set: ancestor = an_ancestor
			descendant_set: descendant = a_descendant
			universe_set: universe = a_universe
		end

feature {ET_AST_NODE} -- Processing
	process_bang_instruction (an_instruction: ET_BANG_INSTRUCTION) is
			-- Replace target, creation type and call by their modified form
		local
			target: ET_WRITABLE
			type: ET_TYPE
			creation_call: ET_QUALIFIED_CALL
			qualified_name: ET_QUALIFIED_FEATURE_NAME
			name: ET_FEATURE_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
			c1, c2: ET_CLASS
			nested_processor: like Current
		do
			if an_instruction.type /= Void then
				good_type (an_instruction.type, ancestor, universe).process (Current)
				type ?= modified_node
				check type /= Void end
			else
				an_instruction.target.process (ancestor_type_evaluator)
				type := ancestor_type_evaluator.named_type
			end

			an_instruction.target.process (Current)
			target ?= modified_node
			check target /= Void end

			c1 := type.type.base_class (ancestor, universe)
			c2 := type.type.base_class (descendant, universe)

			if an_instruction.creation_call /= Void then
				if c1 /= c2 then
					create nested_processor.make (Void, Void, c1, c2, universe)
					an_instruction.creation_call.qualified_name.process (nested_processor)
					qualified_name ?= nested_processor.modified_node
					check qualified_name /= Void end
				else
					qualified_name := an_instruction.creation_call.qualified_name
				end

				args := an_instruction.creation_call.arguments
				if args /= Void then
					args.process (Current)
					args ?= modified_node
					check modified_node /= Void implies args /= Void end
				end
				create creation_call.make (qualified_name, args)
			else
				if c1 /= c2 then
					create nested_processor.make (Void, Void, c1, c2, universe)
					tokens.default_create_feature_name.process (nested_processor)
					name ?= nested_processor.modified_node
					check name /= Void end
					if not name.same_feature_name (tokens.default_create_feature_name) then
						create creation_call.make (name, Void)
					end
				end
			end

			create {ET_BANG_INSTRUCTION} modified_node.make (type, target, creation_call)
		end

	process_bracket_expression (an_expression: ET_BRACKET_EXPRESSION) is
			-- Replace target, brackets and arguments with their modified form
		local
			target1, target2: ET_EXPRESSION
			name: ET_CALL_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
			nested_processor: like Current
		do
			target1 := an_expression.target
			target1.process (ancestor_type_evaluator)
			target1.process (Current)
			target2 ?= modified_node
			check target2 /= Void end
			target2.process (descendant_type_evaluator)

			name := an_expression.name
			if ancestor_type_evaluator.base_class /= descendant_type_evaluator.base_class then
				create nested_processor.make (Void, Void, ancestor_type_evaluator.base_class, descendant_type_evaluator.base_class, universe)
				nested_processor.process_call_name (name)
				name ?= nested_processor.modified_node
				check name /= Void end
			end

			args := argument_list (an_expression.arguments)
			if args /= Void then
				args.process (Current)
				args ?= modified_node
				check modified_node /= Void implies args /= Void end
			end

			modified_node := call_expression (target2, name, args)
		end

	process_call_agent (an_expression: ET_CALL_AGENT) is
			-- Replace target, name and arguments with their modified form
		local
			target1, target2: ET_AGENT_TARGET
			qualified_name: ET_QUALIFIED_FEATURE_NAME
			args: ET_AGENT_ARGUMENT_OPERAND_LIST
			nested_processor: like Current
		do
			target1 := an_expression.target
			if an_expression.is_qualified_call then
				target1.process (ancestor_type_evaluator)

				target1.process (Current)
				target2 ?= modified_node
				check target2 /= Void end
				target2.process (descendant_type_evaluator)

				if ancestor_type_evaluator.base_class /= descendant_type_evaluator.base_class then
					create nested_processor.make (Void, Void, ancestor_type_evaluator.base_class, descendant_type_evaluator.base_class, universe)
					an_expression.qualified_name.process (nested_processor)
					qualified_name ?= nested_processor.modified_node
					check qualified_name /= Void end
				else
					qualified_name := an_expression.qualified_name
				end
			else
				an_expression.qualified_name.process (Current)
				qualified_name ?= modified_node
				check qualified_name /= Void end
			end

			args ?= an_expression.arguments
			if args /= Void then
				args.process (Current)
				args ?= modified_node
				check modified_node /= Void implies args /= Void end
			end

			create {ET_CALL_AGENT} modified_node.make (target2, qualified_name, args)
		end

	process_call_expression (an_expression: ET_CALL_EXPRESSION) is
			-- Replace target, name and arguments with their modified form
		local
			target1, target2: ET_EXPRESSION
			name: ET_FEATURE_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
			nested_processor: like Current
		do
			target1 := an_expression.target
			if target1 /= Void then
				target1.process (ancestor_type_evaluator)

				target1.process (Current)
				target2 ?= modified_node
				check modified_node /= Void implies target2 /= Void end
				target2.process (descendant_type_evaluator)

				if ancestor_type_evaluator.base_class /= descendant_type_evaluator.base_class then
					create nested_processor.make (Void, Void, ancestor_type_evaluator.base_class, descendant_type_evaluator.base_class, universe)
					an_expression.name.process (nested_processor)
					name ?= nested_processor.modified_node
					check name /= Void end
				else
					name := an_expression.name
				end
			else
				an_expression.name.process (Current)
				name ?= modified_node
				check name /= Void end
			end

			args := an_expression.arguments
			if args /= Void then
				args.process (Current)
				args ?= modified_node
				check modified_node /= Void implies args /= Void end
			end

			modified_node := call_expression (target2, name, args)
		end

	process_call_instruction (an_instruction: ET_CALL_INSTRUCTION) is
			-- Replace target, name and arguments with their modified form
		local
			target1, target2: ET_EXPRESSION
			name: ET_FEATURE_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
			nested_processor: like Current
		do
			target1 := an_instruction.target
			if target1 /= Void then
				target1.process (ancestor_type_evaluator)
				target1.process (Current)
				target2 ?= modified_node
				check modified_node /= Void implies target2 /= Void end
				target2.process (descendant_type_evaluator)

				if ancestor_type_evaluator.base_class /= descendant_type_evaluator.base_class then
					create nested_processor.make (Void, Void, ancestor_type_evaluator.base_class, descendant_type_evaluator.base_class, universe)
					an_instruction.name.process (nested_processor)
					name ?= nested_processor.modified_node
					check name /= Void end
				else
					name := an_instruction.name
				end
			else
				an_instruction.name.process (Current)
				name ?= modified_node
				check name /= Void end
			end

			args := an_instruction.arguments
			if args /= Void then
				args.process (Current)
				args ?= modified_node
				check modified_node /= Void implies args /= Void end
			end

			create {ET_CALL_INSTRUCTION} modified_node.make (target2, name, args)
		end

	process_create_expression (an_expression: ET_CREATE_EXPRESSION) is
			-- Replace creation type and call by their modified form
		local
			type: ET_TARGET_TYPE
			creation_call: ET_QUALIFIED_CALL
			qualified_name: ET_QUALIFIED_FEATURE_NAME
			name: ET_FEATURE_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
			c1, c2: ET_CLASS
			nested_processor: like Current
		do
			good_type (an_expression.creation_type, ancestor, universe).process (Current)
			type ?= modified_node
			check type /= Void end

			c1 := type.type.base_class (ancestor, universe)
			c2 := type.type.base_class (descendant, universe)

			if an_expression.creation_call /= Void then
				if c1 /= c2 then
					create nested_processor.make (Void, Void, c1, c2, universe)
					an_expression.creation_call.qualified_name.process (nested_processor)
					qualified_name ?= nested_processor.modified_node
					check qualified_name /= Void end
				else
					qualified_name := an_expression.creation_call.qualified_name
				end

				args := an_expression.creation_call.arguments
				if args /= Void then
					args.process (Current)
					args ?= modified_node
					check modified_node /= Void implies args /= Void end
				end
				create creation_call.make (qualified_name, args)
			else
				if c1 /= c2 then
					create nested_processor.make (Void, Void, c1, c2, universe)
					tokens.default_create_feature_name.process (nested_processor)
					name ?= nested_processor.modified_node
					check name /= Void end
					if not name.same_feature_name (tokens.default_create_feature_name) then
						create creation_call.make (name, Void)
					end
				end
			end

			create {ET_CREATE_EXPRESSION} modified_node.make (type, creation_call)
		end

	process_create_instruction (an_instruction: ET_CREATE_INSTRUCTION) is
			-- Replace target, creation type and call by their modified form
		local
			target1, target2: ET_WRITABLE
			type: ET_TARGET_TYPE
			creation_call: ET_QUALIFIED_CALL
			qualified_name: ET_QUALIFIED_FEATURE_NAME
			name: ET_FEATURE_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
			c1, c2: ET_CLASS
			nested_processor: like Current
		do
			target1 := an_instruction.target
			target1.process (Current)
			target2 ?= modified_node
			check target2 /= Void end

			if an_instruction.creation_type /= Void then
				good_type (an_instruction.creation_type, ancestor, universe).process (Current)
				type ?= modified_node
				check type /= Void end
				c1 := type.type.base_class (ancestor, universe)
				c2 := type.type.base_class (descendant, universe)
			else
				target1.process (ancestor_type_evaluator)
				target2.process (descendant_type_evaluator)
				c1 := ancestor_type_evaluator.base_class
				c2 := descendant_type_evaluator.base_class
			end

			if an_instruction.creation_call /= Void then
				if c1 /= c2 then
					create nested_processor.make (Void, Void, c1, c2, universe)
					an_instruction.creation_call.qualified_name.process (nested_processor)
					qualified_name ?= nested_processor.modified_node
					check qualified_name /= Void end
				else
					qualified_name := an_instruction.creation_call.qualified_name
				end

				args := an_instruction.creation_call.arguments
				if args /= Void then
					args.process (Current)
					args ?= modified_node
					check modified_node /= Void implies args /= Void end
				end
				create creation_call.make (qualified_name, args)
			else
				if c1 /= c2 then
					create nested_processor.make (Void, Void, c1, c2, universe)
					tokens.default_create_feature_name.process (nested_processor)
					name ?= nested_processor.modified_node
					check name /= Void end
					if not name.same_feature_name (tokens.default_create_feature_name) then
						create creation_call.make (name, Void)
					end
				end
			end

			create {ET_CREATE_INSTRUCTION} modified_node.make (type, target2, creation_call)
		end

	process_identifier (an_identifier: ET_IDENTIFIER) is
			-- Rename `an_identifier' if is a renamed feature
		do
			process_call_name (an_identifier)
		end

	process_infix_expression (an_expression: ET_INFIX_EXPRESSION) is
			-- Replace left, right and operator with their modified form
		local
			left1, left2, right: ET_EXPRESSION
			name: ET_CALL_NAME
			nested_processor: like Current
		do
			left1 := an_expression.left
			left1.process (ancestor_type_evaluator)
			left1.process (Current)
			left2 ?= modified_node
			check left2 /= Void end
			left2.process (descendant_type_evaluator)

			name := an_expression.name
			if ancestor_type_evaluator.base_class /= descendant_type_evaluator.base_class then
				create nested_processor.make (Void, Void, ancestor_type_evaluator.base_class, descendant_type_evaluator.base_class, universe)
				nested_processor.process_call_name (name)
				name ?= nested_processor.modified_node
				check name /= Void end
			end

			an_expression.right.process (Current)
			right ?= modified_node
			check right /= Void end

			modified_node := call_expression (left2, name, single_actual (right))
		end

	process_prefix_expression (an_expression: ET_PREFIX_EXPRESSION) is
			-- Replace operator and expression with their modified form
		local
			expr1, expr2: ET_EXPRESSION
			name: ET_CALL_NAME
			nested_processor: like Current
		do
			expr1 := an_expression.expression
			expr1.process (ancestor_type_evaluator)
			expr1.process (Current)
			expr2 ?= modified_node
			check expr2 /= Void end
			expr2.process (descendant_type_evaluator)

			name := an_expression.name
			if ancestor_type_evaluator.base_class /= descendant_type_evaluator.base_class then
				create nested_processor.make (Void, Void, ancestor_type_evaluator.base_class, descendant_type_evaluator.base_class, universe)
				nested_processor.process_call_name (name)
				name ?= nested_processor.modified_node
				check name /= Void end
			end

			modified_node := call_expression (expr2, name, Void)
		end

	process_static_call_expression (an_expression: ET_STATIC_CALL_EXPRESSION) is
			-- Replace type, name and args with their modified form
		local
			type: ET_TARGET_TYPE
			c1, c2: ET_CLASS
			nested_processor: like Current
			qualified_name: ET_QUALIFIED_FEATURE_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
		do
			good_type (an_expression.static_type, ancestor, universe).process (Current)
			type ?= modified_node
			check type /= Void end

			c1 := type.type.base_class (ancestor, universe)
			c2 := type.type.base_class (descendant, universe)
			if c1 /= c2 then
				create nested_processor.make (Void, Void, c1, c2, universe)
				an_expression.qualified_name.process (nested_processor)
				qualified_name ?= nested_processor.modified_node
				check qualified_name /= Void end
			else
				qualified_name := an_expression.qualified_name
			end

			args := an_expression.arguments
			if args /= Void then
				args.process (Current)
				args ?= modified_node
				check modified_node /= Void implies args /= Void end
			end

			create {ET_STATIC_CALL_EXPRESSION} modified_node.make (type, qualified_name, args)
		end

	process_static_call_instruction (an_instruction: ET_STATIC_CALL_INSTRUCTION) is
			-- Replace type, name and args with their modified form
		local
			type: ET_TARGET_TYPE
			c1, c2: ET_CLASS
			nested_processor: like Current
			qualified_name: ET_QUALIFIED_FEATURE_NAME
			args: ET_ACTUAL_ARGUMENT_LIST
		do
			good_type (an_instruction.static_type, ancestor, universe).process (Current)
			type ?= modified_node
			check type /= Void end

			c1 := type.type.base_class (ancestor, universe)
			c2 := type.type.base_class (descendant, universe)
			if c1 /= c2 then
				create nested_processor.make (Void, Void, c1, c2, universe)
				an_instruction.qualified_name.process (nested_processor)
				qualified_name ?= nested_processor.modified_node
				check qualified_name /= Void end
			else
				qualified_name := an_instruction.qualified_name
			end

			args := an_instruction.arguments
			if args /= Void then
				args.process (Current)
				args ?= modified_node
				check modified_node /= Void implies args /= Void end
			end

			create {ET_STATIC_CALL_INSTRUCTION} modified_node.make (type, qualified_name, args)
		end

feature {CI_FEATURE_RENAMER} -- Processing
	process_call_name (a_name: ET_CALL_NAME) is
			-- Reanme `a_name' if needed
		local
			f, g: ET_FEATURE
		do
			f := ancestor.named_feature (a_name)
			if f /= Void then
				g := seeded_feature (descendant, f.first_seed)
				check g /= Void end
				modified_node := g.name
			else
				modified_node := a_name
			end
		end

feature {NONE} -- Implementation
	ancestor_feature: ET_FEATURE
			-- Surrounding feature in `ancestor' (may be Void)

	descendant_feature: ET_FEATURE
			-- Surrounding feature in `descendant' (may be Void)

	ancestor: ET_CLASS
			-- Ancestor class

	descendant: ET_CLASS
			-- descendant class

	ancestor_type_evaluator: CI_EXPRESSION_TYPE_EVALUATOR
			-- Type evaluator inside `ancestor'

	descendant_type_evaluator: CI_EXPRESSION_TYPE_EVALUATOR
			-- type evaluator inside `descendant'

	call_expression (a_target: ET_EXPRESSION; a_name: ET_CALL_NAME; an_args: ET_ACTUAL_ARGUMENT_LIST): ET_EXPRESSION is
			-- Create call of form dependent on `a_name' kind
		local
			feature_name: ET_FEATURE_NAME
		do
			if a_name.is_identifier then
				feature_name ?= a_name
				create {ET_CALL_EXPRESSION} Result.make (a_target, feature_name, an_args)
			elseif a_name.is_infix then
				check an_args.count = 1 end
				create {ET_INFIX_EXPRESSION} Result.make (a_target, infix_operator (a_name), an_args.actual_argument (1))
			elseif a_name.is_prefix then
				check an_args = Void end
				create {ET_PREFIX_EXPRESSION} Result.make (prefix_operator (a_name), a_target)
			elseif a_name.is_bracket then
				create {ET_BRACKET_EXPRESSION} result.make (a_target, create {ET_BRACKET_SYMBOL}.make, bracket_argument_list (an_args))
			end
		end

invariant
	ancestor_exists: ancestor /= Void
	descendant_exists: descendant /= Void
	ancestor_type_evaluator_exists: ancestor_type_evaluator /= Void
	descendant_type_evaluator_exists: descendant_type_evaluator /= Void
end
