indexing
	description: "Objects that change argument and local names of inline agents when they are put into different context,%
		% so that all identifiers stay unique"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_INLINE_AGENT_IDENTIFIER_PROCESSOR

inherit
	CI_AST_MODIFIER
		rename
			make as make_with_universe,
			context as surrounding_class
		export {NONE}
			make_with_universe
		redefine
			surrounding_class,
			process_do_function_inline_agent,
			process_do_procedure_inline_agent,
			process_external_function_inline_agent,
			process_external_procedure_inline_agent,
			process_once_function_inline_agent,
			process_once_procedure_inline_agent,
			process_identifier
		end

create
	make

feature -- Initialization
	make (a_closure: ET_ROUTINE; a_class: like surrounding_class; a_universe: like universe) is
			-- Create processor for placing agents inside `a_closure' in `a_class' within `a_universe'
		require
			a_class_exists: a_class /= Void
			a_universe_exists: a_universe /= Void
		do
			make_with_universe (a_universe)
			create surrounding_closure.make
			surrounding_closure.put (a_closure)
			surrounding_class := a_class
			create renames.make_equal (10)
		ensure
			surrounding_closure_set: surrounding_closure.item = a_closure
			surrounding_class_set: surrounding_class = a_class
			universe_set: universe = a_universe
		end

	reset is
			-- Reset processor between processing different (not nested) agents
		do
			renames.wipe_out
		end


feature {ET_AST_NODE} -- Processing
	process_do_function_inline_agent (an_expression: ET_DO_FUNCTION_INLINE_AGENT) is
			-- Rename agent formals and locals if needed, process agent body and actuals
		local
			compound, rescue_clause: ET_COMPOUND
			actuals: ET_AGENT_ARGUMENT_OPERAND_LIST
			res: ET_DO_FUNCTION_INLINE_AGENT
		do
			process_formals_and_locals (an_expression)
			create res.make (formals, an_expression.type, an_expression.actual_arguments)
			res.set_locals (locals)
			res.set_compound (an_expression.compound)
			res.set_rescue_clause (an_expression.rescue_clause)

			surrounding_closure.put (res)
			-- Note: preconditions and postconditions are not processed because they are useless in instrumented system
			compound := res.compound
			if compound /= Void then
				compound.process (Current)
				compound ?= modified_node
				check modified_node /= Void implies compound /= Void end
			end
			rescue_clause := res.rescue_clause
			if rescue_clause /= Void then
				rescue_clause.process (Current)
				rescue_clause ?= modified_node
				check modified_node /= Void implies rescue_clause /= Void end
			end
			surrounding_closure.remove

			actuals ?= res.actual_arguments
			if actuals /= Void then
				actuals.process (Current)
				actuals ?= modified_node
				check modified_node /= Void implies actuals /= Void end
			end

			res.set_actual_arguments (actuals)
			res.set_compound (compound)
			res.set_rescue_clause (rescue_clause)
			modified_node := res
		end

	process_do_procedure_inline_agent (an_expression: ET_DO_PROCEDURE_INLINE_AGENT) is
			-- Rename agent formals and locals if needed, process agent body and actuals
		local
			compound, rescue_clause: ET_COMPOUND
			actuals: ET_AGENT_ARGUMENT_OPERAND_LIST
			res: ET_DO_PROCEDURE_INLINE_AGENT
		do
			process_formals_and_locals (an_expression)
			create res.make (formals, an_expression.actual_arguments)
			res.set_locals (locals)
			res.set_compound (an_expression.compound)
			res.set_rescue_clause (an_expression.rescue_clause)

			surrounding_closure.put (res)
			-- Note: preconditions and postconditions are not processed because they are useless in instrumented system
			compound := res.compound
			if compound /= Void then
				compound.process (Current)
				compound ?= modified_node
				check modified_node /= Void implies compound /= Void end
			end
			rescue_clause := res.rescue_clause
			if rescue_clause /= Void then
				rescue_clause.process (Current)
				rescue_clause ?= modified_node
				check modified_node /= Void implies rescue_clause /= Void end
			end
			surrounding_closure.remove

			actuals ?= res.actual_arguments
			if actuals /= Void then
				actuals.process (Current)
				actuals ?= modified_node
				check modified_node /= Void implies actuals /= Void end
			end

			res.set_actual_arguments (actuals)
			res.set_compound (compound)
			res.set_rescue_clause (rescue_clause)
			modified_node := res
		end

	process_external_function_inline_agent (an_expression: ET_EXTERNAL_FUNCTION_INLINE_AGENT) is
			-- Rename agent formals if needed, process actuals
			-- Note: formals are renamed in Eiffel signature, but it doesn't influence external code
		local
			actuals: ET_AGENT_ARGUMENT_OPERAND_LIST
			res: ET_EXTERNAL_FUNCTION_INLINE_AGENT
		do
			process_formals_and_locals (an_expression)
			create res.make (formals, an_expression.type, an_expression.language, an_expression.actual_arguments)

			actuals ?= res.actual_arguments
			if actuals /= Void then
				actuals.process (Current)
				actuals ?= modified_node
				check modified_node /= Void implies actuals /= Void end
			end

			res.set_actual_arguments (actuals)
			modified_node := res
		end

	process_external_procedure_inline_agent (an_expression: ET_EXTERNAL_PROCEDURE_INLINE_AGENT) is
			-- Rename agent formals if needed, process agentactuals
			-- Note: formals are renamed in Eiffel signature, but it doesn't influence external code			
		local
			actuals: ET_AGENT_ARGUMENT_OPERAND_LIST
			res: ET_EXTERNAL_PROCEDURE_INLINE_AGENT
		do
			process_formals_and_locals (an_expression)
			create res.make (formals, an_expression.language, an_expression.actual_arguments)

			actuals ?= res.actual_arguments
			if actuals /= Void then
				actuals.process (Current)
				actuals ?= modified_node
				check modified_node /= Void implies actuals /= Void end
			end

			res.set_actual_arguments (actuals)
			modified_node := res
		end

	process_once_function_inline_agent (an_expression: ET_ONCE_FUNCTION_INLINE_AGENT) is
			-- Rename agent formals and locals if needed, process agent body and actuals
		local
			compound, rescue_clause: ET_COMPOUND
			actuals: ET_AGENT_ARGUMENT_OPERAND_LIST
			res: ET_ONCE_FUNCTION_INLINE_AGENT
		do
			process_formals_and_locals (an_expression)
			create res.make (formals, an_expression.type, an_expression.actual_arguments)
			res.set_locals (locals)
			res.set_compound (an_expression.compound)
			res.set_rescue_clause (an_expression.rescue_clause)

			surrounding_closure.put (res)
			-- Note: preconditions and postconditions are not processed because they are useless in instrumented system
			compound := res.compound
			if compound /= Void then
				compound.process (Current)
				compound ?= modified_node
				check modified_node /= Void implies compound /= Void end
			end
			rescue_clause := res.rescue_clause
			if rescue_clause /= Void then
				rescue_clause.process (Current)
				rescue_clause ?= modified_node
				check modified_node /= Void implies rescue_clause /= Void end
			end
			surrounding_closure.remove

			actuals ?= res.actual_arguments
			if actuals /= Void then
				actuals.process (Current)
				actuals ?= modified_node
				check modified_node /= Void implies actuals /= Void end
			end

			res.set_actual_arguments (actuals)
			res.set_compound (compound)
			res.set_rescue_clause (rescue_clause)
			modified_node := res
		end

	process_once_procedure_inline_agent (an_expression: ET_ONCE_PROCEDURE_INLINE_AGENT) is
			-- Rename agent formals and locals if needed, process agent body and actuals
		local
			compound, rescue_clause: ET_COMPOUND
			actuals: ET_AGENT_ARGUMENT_OPERAND_LIST
			res: ET_ONCE_PROCEDURE_INLINE_AGENT
		do
			process_formals_and_locals (an_expression)
			create res.make (formals, an_expression.actual_arguments)
			res.set_locals (locals)
			res.set_compound (an_expression.compound)
			res.set_rescue_clause (an_expression.rescue_clause)

			surrounding_closure.put (res)
			-- Note: preconditions and postconditions are not processed because they are useless in instrumented system
			compound := res.compound
			if compound /= Void then
				compound.process (Current)
				compound ?= modified_node
				check modified_node /= Void implies compound /= Void end
			end
			rescue_clause := res.rescue_clause
			if rescue_clause /= Void then
				rescue_clause.process (Current)
				rescue_clause ?= modified_node
				check modified_node /= Void implies rescue_clause /= Void end
			end
			surrounding_closure.remove

			actuals ?= res.actual_arguments
			if actuals /= Void then
				actuals.process (Current)
				actuals ?= modified_node
				check modified_node /= Void implies actuals /= Void end
			end

			res.set_actual_arguments (actuals)
			res.set_compound (compound)
			res.set_rescue_clause (rescue_clause)
			modified_node := res
		end

	process_identifier (an_identifier: ET_IDENTIFIER) is
			-- If `an_identifier' was renamed, replace it with new one
		do
			renames.search (an_identifier.name)
			if renames.found then
				create {ET_IDENTIFIER} modified_node.make (renames.found_item)
			else
				modified_node := an_identifier
			end
		end

feature {NONE} -- Implementation
	surrounding_closure: CI_CLOSURE_STACK
			-- Closure, inside which agent has to be placed

	surrounding_class: ET_CLASS
			-- Class, inside which inline agent has to be placed

	renames: DS_HASH_TABLE [STRING, STRING]
			-- Table that masp old identifier names to new ones

	unique_identifier (an_identifier: ET_IDENTIFIER): ET_IDENTIFIER is
			-- Identifier made from `an_identifier' that is unique in current context
		require
			an_identifier_exists: an_identifier /= Void
		do
			from
				Result := an_identifier.deep_twin
			until
				not has_name_clash (Result)
			loop
				Result.name.append_character ('_')
				create Result.make (Result.name)
			end
		ensure
			result_exists: Result /= Void
		end

	has_name_clash (an_identifier: ET_IDENTIFIER): BOOLEAN is
			-- Does `an_identifier' has name clash in current context?
			-- (does its name conflict with features of `aurrounding_class' or arguments and locals of `surrounding_routine'?)
		require
			an_identifier_exists: an_identifier /= Void
		do
			if surrounding_class.named_feature (an_identifier) /= Void then
				Result := True
			elseif formal_by_name (surrounding_closure.arguments, an_identifier) /= Void then
				Result := True
			elseif local_by_name (surrounding_closure.locals, an_identifier) /= Void then
				Result := True
			end
		end

	formals: ET_FORMAL_ARGUMENT_LIST
			-- Modified formal arguments

	locals: ET_LOCAL_VARIABLE_LIST
			-- Modified local variables

	process_formals_and_locals (an_agent: ET_INLINE_AGENT) is
			-- Rename formals and locals if needed
		local
			i, nb: INTEGER
			id: ET_IDENTIFIER
			arg: ET_FORMAL_ARGUMENT
			var: ET_LOCAL_VARIABLE
		do
			if an_agent.formal_arguments /= Void then
				nb := an_agent.formal_arguments.count
				create formals.make_with_capacity (nb)
				from i := nb until i < 1 loop
					arg := an_agent.formal_arguments.formal_argument (i)
					id := unique_identifier (arg.name)
					if not id.same_identifier (arg.name) then
						renames.force_new (id.name, arg.name.name)
					end
					formals.put_first (create {ET_FORMAL_ARGUMENT}.make (id, arg.declared_type))
					i := i - 1
				end
			end

			if an_agent.locals /= Void then
				nb := an_agent.locals.count
				create locals.make_with_capacity (nb)
				from i := nb until i < 1 loop
					var := an_agent.locals.local_variable (i)
					id := unique_identifier (var.name)
					if not id.same_identifier (var.name) then
						renames.force_new (id.name, var.name.name)
					end
					locals.put_first (create {ET_LOCAL_VARIABLE}.make (id, var.declared_type))
					i := i - 1
				end
			end
		end
invariant
	surrounding_closure_exists: surrounding_closure /= Void
	surrounding_class_exists: surrounding_class /= Void
	renames_exists: renames /= Void
end
