indexing
	description: "Eiffel types of trackable variables"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_VARIABLE_TYPE

inherit
	CI_PRINTABLE_CLASSES

	CI_CODE_GENERATION
		export {NONE}
			all
		end

create
	make,
	make_string_value

feature -- Initialization
	make (a_type: ET_TYPE; a_context: like context; a_universe: like universe) is
			-- Create type information for variable with declared type `a_type', which appears in `a_context' in `a_universe'
		require
			a_type_exists: a_type /= Void
			a_context_exists: a_context /= Void
			a_universe_exists: a_universe /= Void
		do
			universe := a_universe
			context := a_context

			named_type ?= good_type (a_type.named_type (context, universe), context, universe)
			check named_type /= Void end

			base_type := a_type.base_type (context, universe) --named_type.base_type (context, universe)

			if a_type.is_formal_type (context, universe) then
				create {CI_UNKNOWN} kind.make (instrumented_class.name, universe)
			elseif is_string (base_class) then
				create {CI_STRING_REFERENCE} kind.make (universe)
			elseif is_printable (base_class) then -- Other than string
				create {CI_PRINTABLE} kind.make (universe)
			elseif base_type.is_expanded then
				create {CI_EXPANDED} kind.make (base_class.name, instrumented_class.name, universe)
			elseif has_expanded_or_printable_descendants then
				create {CI_UNKNOWN} kind.make (instrumented_class.name, universe)
			else
				create {CI_REFERENCE} kind.make (universe)
			end
		ensure
			universe_set: universe = a_universe
			context_set: context = a_context
		end

	make_string_value (a_class: ET_CLASS; a_context: like context; a_universe: like universe) is
			-- Create type information for string value variable, which appears in `a_context' in `a_universe'
		require
			a_class_exists: a_class /= Void
			a_class_is_string: is_string (a_class)
			a_context_exists: a_context /= Void
			a_universe_exists: a_universe /= Void
		do
			universe := a_universe
			context := a_context
			base_type := type_from_nongeneric_class (a_class)
			named_type := base_type
			create {CI_PRINTABLE} kind.make (universe)
		ensure
			universe_set: universe = a_universe
			context_set: context = a_context
		end

feature -- Access
	kind: CI_VARIABLE_TYPE_KIND
			-- Type kind

	named_type: ET_NAMED_TYPE
			-- Variable named type

	base_type: ET_BASE_TYPE
			-- Variable base type

	base_class: ET_CLASS is
			-- Variable base class
		do
			Result := base_type.direct_base_class (universe)
		end

	context: ET_TYPE_CONTEXT
			-- Context in which variable appears

	surrounding_class: ET_CLASS is
			-- Class in which variable appears (base class of context)
		do
			Result := context.base_class (universe)
		end

	instrumented_class: ET_CLASS is
			-- Class in which variable information is used (root class of context)
		do
			Result := context.root_context.direct_base_class (universe)
		end

	representation_type: STRING is
			-- Name of Daikon representation type for current type
		do
			Result := kind.representation_type (base_class.name)
		end

	universe: ET_UNIVERSE
			-- Universe in which variable appears

feature {NONE} -- Implementation
	has_expanded_or_printable_descendants: BOOLEAN is
			-- Does `base_class' have expanded or printable descendants?
		local
			descendants: DS_ARRAYED_LIST [ET_CLASS]
			c: ET_CLASS
			i: INTEGER
		do
			descendants := base_class.descendants (universe)
			from
				i := 1
			until
				i > descendants.count
			loop
				c := descendants.item (i)
				if c.is_expanded or is_printable (c) then
					Result := True
					i := descendants.count	-- Jump out of the loop
				end
				i := i + 1
			end
		end

invariant
	kind_exists: kind /= Void
	universe_exists: universe /= Void
	equal_universes: universe = kind.universe
	context_exists: context /= Void
	base_type_exists: base_type /= Void
	named_type_exists: named_type /= Void
	base_class_exists: base_class /= Void
	surrounding_class_exists: surrounding_class /= Void
	instrumented_class_exists: instrumented_class /= Void
	representation_type_not_empty: not representation_type.is_empty
end
