indexing
	description	: "Contract Inference Tool that Applies Daikon to Eiffel Language"
	date: "$Date$"
	revision: "$Revision$"

class
	CITADEL

inherit
	CI_SETTINGS_CLIENT

	CI_CODE_GENERATION

	CI_EXCLUDED_ITEMS

	KL_SHARED_FILE_SYSTEM

	KL_SHARED_STRING_EQUALITY_TESTER

	UT_SHARED_ISE_VERSIONS
		export {NONE} all end

	EIFFEL_ENV
create
	execute

feature -- Execution

	execute is
			-- Execute CITADEL
		local
			command_line_parser: CITADEL_COMMAND_LINE_PARSER
			root_class_name, root_feature_name: ET_IDENTIFIER
			ecf_parser: ET_ECF_PARSER
			ast_factory: ET_DECORATED_AST_FACTORY
			ppt_printer: CI_DAIKON_PROGRAM_POINT_PRINTER
			c: ET_CLASS
			compiler_error_handler: ET_ERROR_HANDLER
			universe_storer: CI_STORER [ET_UNIVERSE]
		do
			create error_handler.make_standard
			error_handler.report_start_up

			-- Configure settings
			create command_line_parser.make_null
			command_line_parser.set_error_handler (error_handler)
			command_line_parser.process_arguments
			if error_handler.has_error then
				error_handler.terminate
			end

			-- Check if all prerequisites are met before any processing
			if Settings.compilation_enabled or initial_system_compilation_needed then
				check_environment_variable
				set_precompile (False)
			end
			if Settings.inference_enabled then
				create daikon.make_null
				daikon.set_error_handler (error_handler)
				daikon.check_environment_variables
			end
			if error_handler.has_error then
				error_handler.terminate
			end

			create_output_directories

			if initial_system_compilation_needed then
				error_handler.report_initial_system_compilation
				create compiler_error_handler.make_null
				compiler_error_handler.set_error_file (error_handler.error_file)
				compiler_error_handler.set_info_file (error_handler.info_file)
				compiler_error_handler.set_warning_file (error_handler.warning_file)
				compiler_error_handler.set_verbose (error_handler.is_verbose)

				-- Load a universe from .ecf file				
				create ecf_parser.make (compiler_error_handler)
				if Settings.system_target /= Void then
					ecf_parser.set_target (Settings.system_target)
				end
				ecf_parser.set_workbench_build (Settings.workbench_mode)
				ecf_parser.set_compiler_version (ise_latest)
				ecf_parser.load (Settings.system_name)
				if ecf_parser.is_error then
					error_handler.terminate
				else
					universe := ecf_parser.last_universe
					system := ecf_parser.last_system

					-- Setup root class and root creation procedure (Is there a way to do it better?)
					create root_class_name.make (ecf_parser.last_system.application_target.root.class_type_name)
					universe.set_root_class (root_class_name)
					create root_feature_name.make (ecf_parser.last_system.application_target.root.feature_name)
					universe.set_root_creation (root_feature_name)

					-- Change AST factory, ET_AST_ITERATOR doesn't work properly otherwise (Why?)
					create ast_factory.make
					ast_factory.set_keep_all_breaks (True)
					universe.set_ast_factory (ast_factory)

					universe.set_ise_version (ise_latest)

					-- Compile system
					universe.compile_system
					-- After the first time some classes in system are not compiler properly
					universe.compile_system

					if universe.error_handler.has_eiffel_error then
						error_handler.report_original_system_compilation_failed
						error_handler.terminate
					end
				end
			end
			create instrumented_system.make_null
			instrumented_system.set_error_handler (error_handler)

			if Settings.instrumentation_enabled then
				chose_classes_to_instrument
				if not Settings.classes_to_instrument.is_empty then
					error_handler.report_instrumentation
					if Settings.annotation_enabled then
						create universe_storer
						universe_storer.store_item (universe, Settings.original_universe_file_name)
					end

					-- Generate utility classes
					generate_extra_classes

					create ppt_printer
					open_instrumentation_output
					ppt_printer.set_static_output (instrumentation_output)
					create instrumenter.make (universe, ppt_printer)
					instrumenter.set_error_handler (error_handler)
					-- Instrument system and print classes that have changed
					instrumenter.process_system
					from
						Settings.extra_changed_classes.start
					until
						Settings.extra_changed_classes.after
					loop
						c := universe.classes.item (Settings.extra_changed_classes.item_for_iteration)
						instrumented_system.print_class (c, universe)
						Settings.extra_changed_classes.forth
					end
					-- Print instrumented classes
					from
						Settings.classes_to_instrument.start
					until
						Settings.classes_to_instrument.after
					loop
						c := universe.classes.item (Settings.classes_to_instrument.item_for_iteration)
						instrumented_system.print_class (c, universe)
						Settings.classes_to_instrument.forth
					end
					instrumentation_output.close

					instrumented_system.generate_configuration_file (system)
				else
					error_handler.report_no_classes_to_instrument
				end
				if error_handler.has_error then
					error_handler.terminate
				end
			end
			if Settings.compilation_enabled then
				error_handler.report_compilation
				instrumented_system.compile
				if error_handler.has_error then
					error_handler.terminate
				end
			end
			if Settings.execution_enabled then
				error_handler.report_execution
				instrumented_system.execute
				if error_handler.has_error then
					error_handler.terminate
				end
			end
			if Settings.inference_enabled then
				error_handler.report_inference
				daikon.execute
				if error_handler.has_error then
					error_handler.terminate
				end
				error_handler.report_printing
				daikon.print_invariants
				if error_handler.has_error then
					error_handler.terminate
				end
			end
			if Settings.annotation_enabled then
				error_handler.report_annotation
				if Settings.instrumentation_enabled then
					universe_storer.retrieve_item (Settings.original_universe_file_name)
					universe := universe_storer.item
				end

				create ppt_printer
				create annotator.make (universe, ppt_printer)
				annotator.set_error_handler (error_handler)
				annotator.read_assertions
				if error_handler.has_error then
					error_handler.terminate
				end
				annotator.annotate_classes
				annotator.print_annotated_classes
			end

			delete_temporary_files
			error_handler.report_exiting
			io.read_character
		end

feature {NONE} -- Implementation
	application_name: STRING is "CITADEL"
			-- Application name

	create_output_directories is
			-- Create directories for CITADEL's output
		once
			file_system.recursive_create_directory (Settings.output_path)
			if Settings.instrumentation_enabled then
				file_system.recursive_delete_directory (Settings.instrumented_cluster_path)
				file_system.recursive_create_directory (Settings.instrumented_cluster_path)
			end
			if Settings.instrumentation_enabled or Settings.execution_enabled then
				file_system.recursive_create_directory (Settings.daikon_files_path)
				if Settings.instrumentation_enabled then
					file_system.delete_file (Settings.daikon_static_input_name)
				end
				file_system.delete_file (Settings.daikon_runtime_input_name)
				file_system.delete_file (Settings.daikon_output_name)
				file_system.delete_file (Settings.invariants_name)
			end
			if Settings.annotation_enabled then
				file_system.recursive_delete_directory (Settings.annotated_system_path)
				file_system.recursive_create_directory (Settings.annotated_system_path)
			end
		rescue
			error_handler.report_error_message ("Can't create output directories for reason: " + error_handler.Exceptions.tag_name)
			error_handler.terminate
		end

	delete_temporary_files is
			-- Delete files that aren't needed after execution
		do
			file_system.delete_file (Settings.original_universe_file_name)
		end


	chose_classes_to_instrument is
			-- Add classes to `Settings.classes_to_instrument'
		local
			clusters, added_clusters, clusters_with_specified_classes: DS_HASH_SET [STRING]
			classes: DS_HASH_SET [ET_CLASS_NAME]
			c: ET_CLASS
			cluster: ET_CLUSTER
			full_name: STRING
			new_size: INTEGER
		do
			clusters := Settings.clusters_to_instrument
			if clusters.is_empty and universe.clusters /= Void then
				-- If no clusters were specified, chose all clusters
				new_size := universe.cluster_count
				if new_size > clusters.capacity then
					clusters.resize (new_size)
				end
				add_clusters (clusters, universe.clusters)
			else
				-- If some clusters were specified, check if they exist and add their subclusters
				create added_clusters.make_default
				from
					clusters.start
				until
					clusters.after
				loop
					cluster := universe.cluster_by_name (cluster_name_from_string (clusters.item_for_iteration))
					if cluster = Void then
						error_handler.report_unknown_cluster (clusters.item_for_iteration)
					elseif cluster.subclusters /= Void then
						new_size := clusters.count + cluster.subclusters.count
						if new_size > added_clusters.capacity then
							added_clusters.resize (new_size)
						end
						add_clusters (added_clusters, cluster.subclusters)
					end
					clusters.forth
				end
				new_size := clusters.count + added_clusters.count
				if new_size > clusters.capacity then
					clusters.resize (new_size)
				end
				clusters.extend (added_clusters)
			end

			classes := Settings.classes_to_instrument
			create clusters_with_specified_classes.make_default
			clusters_with_specified_classes.set_equality_tester (case_insensitive_string_equality_tester)
			-- Remove classes that do not exists or are not in system
			-- Fill `clusters_with_specified_classes'
			from
				classes.start
			until
				classes.after
			loop
				universe.classes.search (classes.item_for_iteration)
				if universe.classes.found then
					c := universe.classes.found_item
					if excluded_classes.has_class (c) then
						error_handler.report_wrong_class_instrumented (c.name)
						classes.remove (classes.item_for_iteration)
					end
					if c.in_system then
						if c.group /= Void then
							full_name := c.group.full_name (Settings.subcluster_separator)
							clusters_with_specified_classes.force (full_name)
							classes.forth
						end
					else
						error_handler.report_unknown_class (classes.item_for_iteration)
						classes.remove (classes.item_for_iteration)
					end
				else
					error_handler.report_unknown_class (classes.item_for_iteration)
					classes.remove (classes.item_for_iteration)
				end
			end

			-- Add all classes from instrumented clusters, which don't have specified classes, to instrumented classes
			from
				universe.classes.start
			until
				universe.classes.after
			loop
				c := universe.classes.item_for_iteration
				if c.group /= Void then
					full_name := c.group.full_name (Settings.subcluster_separator)
					if c.in_system and not excluded_classes.has_class (c) and
						clusters.has (full_name) and not clusters_with_specified_classes.has (full_name) then
						classes.force (c.name)
					end
				end
				universe.classes.forth
			end
		end

	cluster_name_from_string (a_string: STRING): ARRAY [STRING] is
			-- Array representation of cluster name made from string representation
		local
			list: LIST [STRING]
			i: INTEGER
		do
			list := a_string.split (Settings.subcluster_separator)
			create Result.make (1, list.count)
			from
				i := Result.lower
				list.start
			until
				list.after
			loop
				Result.put (list.item_for_iteration, i)
				i := i + 1
				list.forth
			end
		end

	add_clusters (res: DS_HASH_SET [STRING]; source: ET_CLUSTERS) is
			-- Add names of clusters from `source' and their subclusters to `res'
		require
			res_exists: res /= Void
			res_large_enough: source /= Void implies res.extendible (source.count)
		local
			cluster: ET_CLUSTER
		do
			if source /= Void then
				from
					source.clusters.start
				until
					source.clusters.after
				loop
					cluster := source.clusters.item_for_iteration
					res.put (cluster.full_name (Settings.subcluster_separator))
					add_clusters (res, cluster.subclusters)
					source.clusters.forth
				end
			end
		end

	generate_extra_classes is
			-- Generate identities class
		local
			identities_generator: CI_IDENTITIES_GENERATOR
			runtime_output_generator: CI_RUNTIME_OUTPUT_GENERATOR
		do
			create identities_generator.make (universe)
			identities_generator.generate_identities
			instrumented_system.print_class (identities_generator.identities, universe)

			create runtime_output_generator.make (Settings.daikon_runtime_input_name, universe)
			runtime_output_generator.generate_runtime_output
			instrumented_system.print_class (runtime_output_generator.runtime_output, universe)
			runtime_output_generator.generate_runtime_output_client
			instrumented_system.print_class (runtime_output_generator.runtime_output_client, universe)
		end

	initial_system_compilation_needed: BOOLEAN is
			-- Does initial system needs to be compiled?
		once
			-- Yes, if instrumentation or annotation is enabled
			Result := Settings.instrumentation_enabled or Settings.annotation_enabled
		end

	instrumentation_output: PLAIN_TEXT_FILE
			-- Output file for instrumentation phase

	open_instrumentation_output is
			-- Create and open `instrumentation_output'
		do
			create instrumentation_output.make_open_write (Settings.daikon_static_input_name)
		rescue
			error_handler.report_error_message ("Can't create file " + Settings.daikon_static_input_name + " for reason "
				+ error_handler.Exceptions.tag_name)
		end

	universe: ET_UNIVERSE
			-- Universe for a system under processing

	system: CONF_SYSTEM
			-- System under processing

	instrumenter: CI_INSTRUMENTER
			-- Instrumenter

	instrumented_system: CI_INSTRUMENTED_SYSTEM
			-- Instrumented system proxy

	daikon: CI_DAIKON
			-- Daikon invariant detector proxy

	annotator: CI_ANNOTATOR
			-- Annotator

	error_handler: CI_ERROR_HANDLER
			-- Error handler

invariant
	error_handler_exists: error_handler /= Void
	universe_exists_if_needed: initial_system_compilation_needed implies universe /= Void
	system_exists_if_needed: initial_system_compilation_needed implies system /= Void
	instrumenter_exists_if_needed: Settings.instrumentation_enabled implies instrumenter /= Void
	instrumented_system_exists:	instrumented_system /= Void
	daikon_exists_if_needed: Settings.inference_enabled implies daikon /= Void
end
