// ***** This file is automatically generated from SeqIntComparison.java.jpp

package daikon.inv.binary.sequenceScalar;

import daikon.*;
import daikon.derive.unary.*;
import daikon.inv.*;
import daikon.inv.unary.sequence.*;
import daikon.inv.binary.twoSequence.*;
import daikon.suppress.*;

import utilMDE.*;

import java.util.*;
import java.util.logging.Logger;

/**
 * Represents an invariant between a double scalar and a
 * a sequence of double values.
 * Prints as <samp>x[] elements >= y</samp> where <samp>x</samp> is a
 * double sequence and <samp>y</samp> is a double scalar.
 **/
public final class SeqFloatGreaterEqual
  extends SequenceFloat
{
  // We are Serializable, so we specify a version to allow changes to
  // method signatures without breaking serialization.  If you add or
  // remove fields, you should change this number to the current date.
  static final long serialVersionUID = 20030822L;

  // Variables starting with dkconfig_ should only be set via the
  // daikon.config.Configuration interface.
  /**
   * Boolean.  True iff SeqFloatGreaterEqual invariants should be considered.
   **/
  public static boolean dkconfig_enabled = true;

  public static final Logger debug
    = Logger.getLogger("daikon.inv.binary.sequenceScalar.SeqFloatGreaterEqual");

  static boolean debugSeqIntComparison = false;

  protected SeqFloatGreaterEqual(PptSlice ppt) {
    super(ppt);
  }

  private static SeqFloatGreaterEqual proto;

  /** Returns the prototype invariant for SeqFloatGreaterEqual **/
  public static Invariant get_proto () {
    if (proto == null)
      proto = new SeqFloatGreaterEqual (null);
    return (proto);
  }

  /** Returns whether or not this invariant is enabled **/
  public boolean enabled() {
    return dkconfig_enabled;
  }

  /** Non-equal SeqIntComparison is only valid on integral types **/
  public boolean instantiate_ok (VarInfo[] vis) {

    if (!valid_types (vis))
      return (false);

    VarInfo seqvar = null;
    VarInfo sclvar = null;
    if (vis[0].rep_type == ProglangType.DOUBLE_ARRAY) {
      seqvar = vis[0];
      sclvar = vis[1];
    } else {
      seqvar = vis[1];
      sclvar = vis[0];
    }

    Assert.assertTrue(sclvar.rep_type == ProglangType.DOUBLE);
    Assert.assertTrue(seqvar.rep_type == ProglangType.DOUBLE_ARRAY);

    return (true);
  }

  /** instantiates the invariant on the specified slice **/
  protected Invariant instantiate_dyn (PptSlice slice) {
    return new SeqFloatGreaterEqual (slice);
  }

  /**
   * Checks to see if the comparison is obvious statically.  Makes the
   * following checks <pre>
   *
   *    max(A[]) op A[]
   *    min(A[]) op A[]
   * </pre>
   *
   * JHP: Note that these are not strict implications, these are merely
   * uninteresting comparisons (except when op is GreaterEqual for max
   * and LessEqual for min)
   */
  public DiscardInfo isObviousStatically(VarInfo[] vis) {

    SequenceMin seqmin = null;
    SequenceMax seqmax = null;
    VarInfo sclseq = null;
    if (sclvar(vis).derived instanceof SequenceMin) {
      seqmin = (SequenceMin) sclvar(vis).derived;
      sclseq = seqmin.base;
    } else if (sclvar(vis).derived instanceof SequenceMax) {
      seqmax = (SequenceMax) sclvar(vis).derived;
      sclseq = seqmax.base;
    }
    if (seqvar(vis) == sclseq) {
      return new DiscardInfo (this, DiscardCode.obvious,
                              sclvar(vis).name() + " is min/max ");
    }
    return (null);
  }

  public SeqFloatGreaterEqual clone() {
    SeqFloatGreaterEqual result = (SeqFloatGreaterEqual) super.clone();
    return result;
  }

  public String repr() {
    return "SeqFloatGreaterEqual" + varNames() + ": "
      + ",falsified=" + falsified;
  }

  public String format_using(OutputFormat format) {

    if (format.isJavaFamily()) return format_java_family(format);

    if (format == OutputFormat.DAIKON) return format_daikon();
    if (format == OutputFormat.IOA) return format_ioa();
    if (format == OutputFormat.ESCJAVA) return format_esc();
    if (format == OutputFormat.SIMPLIFY) return format_simplify();
    if (format == OutputFormat.EIFFEL) return format_eiffel();      

    return format_unimplemented(format);
  }

  public String format_daikon() {
    return seqvar().name() + " elements >= " + sclvar().name();
  }

  /* IOA */
  public String format_ioa() {
    Quantify.IOAQuantification quant = VarInfo.get_ioa_quantify (seqvar());
    return quant.getQuantifierExp() + quant.getMembershipRestriction(0) +
      " => " + quant.getVarIndexedString(0) + ">=" + sclvar().ioa_name() + quant.getClosingExp();
  }

  public String format_esc() {
    String[] form = VarInfo.esc_quantify (seqvar(), sclvar());
    return form[0] + "(" + form[1] + " >= " + form[2] + ")"
      + form[3];
  }

  public String format_simplify() {
    String[] form = VarInfo.simplify_quantify (seqvar(), sclvar());
    return form[0] + "(>= " + form[1] + " "
      + form[2] + ")" + form[3];
  }

  public String format_java_family(OutputFormat format) {
    return "daikon.Quant.eltsGTE("
      + seqvar().name_using(format) + ", " + sclvar().name_using(format) + ")";
  }

  public String format_eiffel () {

    return seqvar().name() + ".for_all (agent {" + sclvar().type.base() + 
    "}.infix \">=\" (" + sclvar().name() + "))";

  }  

  public InvariantStatus check_modified(double[] a, double x, int count) {
    /*if (logDetail() || debug.isLoggable(Level.FINE))
      log(debug,"(>= " + ArraysMDE.toString(a)
      + " " + x);*/
    for (int i=0; i<a.length; i++) {

        // Assert.assertTrue (seqvar().type.elementIsIntegral());

      if (!(Global.fuzzy.gte (a[i], x)))
        return InvariantStatus.FALSIFIED;
    }
    return InvariantStatus.NO_CHANGE;
  }

  public InvariantStatus add_modified(double[] a, double x, int count) {
    return check_modified(a, x, count);
  }

  protected double computeConfidence() {

    // If there are no samples over our variables, its unjustified
    if (ppt.num_samples() == 0)
      return CONFIDENCE_UNJUSTIFIED;

    // If the array never has any elements, its unjustified
    ValueSet.ValueSetFloatArray vs = (ValueSet.ValueSetFloatArray) seqvar().get_value_set();
    if (vs.elem_cnt() == 0) {
      return CONFIDENCE_UNJUSTIFIED;
    }

      // return 1 - Math.pow(.5, vs.elem_cnt());
      return 1 - Math.pow(.5, ppt.num_samples());
  }

  public boolean isExact() {

      return false;
  }

  public boolean isSameFormula(Invariant other) {
    return true;
  }

  public boolean isExclusiveFormula(Invariant other) {
    return false;
  }

  // Look up a previously instantiated invariant.
  public static SeqFloatGreaterEqual find(PptSlice ppt) {
    Assert.assertTrue(ppt.arity() == 2);
    for (Invariant inv : ppt.invs) {
      if (inv instanceof SeqFloatGreaterEqual)
        return (SeqFloatGreaterEqual) inv;
    }
    return null;
  }

  /**
   * Checks to see if this is obvious over the specified variables.
   * Implements the following checks: <pre>
   *
   *  (x op B[]) ^ (B[] subsequence A[]) ==> (x op A[])
   *  (A[] == [])                        ==> (x op A[])
   *
   */
  public DiscardInfo isObviousDynamically(VarInfo[] vis) {

    DiscardInfo super_result = super.isObviousDynamically(vis);
    if (super_result != null) {
      return super_result;
    }

    VarInfo seqvar = seqvar(vis);
    VarInfo sclvar = sclvar(vis);
    //Debug.log (getClass(), ppt, vis, "Considering  over" + seqvar.name()
    //           + " and " + sclvar.name());

    // Look for the same property over a supersequence of this one.  This
    // doesn't need to explicitly ignore oher members of the equality set
    // because those members won't have any invariants over them.
    PptTopLevel pptt = ppt.parent;
    for (Iterator<Invariant> inv_itor = pptt.invariants_iterator(); inv_itor.hasNext(); ) {
      Invariant inv = inv_itor.next();
      if (inv == this) {
        continue;
      }
      if (inv instanceof SeqFloatGreaterEqual) {
        SeqFloatGreaterEqual other = (SeqFloatGreaterEqual) inv;
        // Fmt.pf ("considering %s seqvar=%s, other=%s", other.format(),
        // seqvar().name(), other.seqvar().name());
        if (pptt.is_subsequence (seqvar(), other.seqvar())
            && (sclvar(vis) == other.sclvar())) {
          // Fmt.pf ("is subsequence");
          return new DiscardInfo(this, DiscardCode.obvious, seqvar().name()
                                 + " is a subsequence of "
                                 + other.seqvar().name() + " and "
                                 + other.format() + " holds");
        }
      }
    }

    // JHP: handled in confidence test now
    // (A[] == []) ==> A[] op x
    if (false) {
      if (pptt.is_empty (seqvar))
        return new DiscardInfo (this, DiscardCode.obvious, "The sequence "
                                + seqvar.name() + " is always empty");
    }

    if (isExact()) {
      return null;
    }

    // JHP: these presume that this invariant is true and should thus be
    // moved to uninteresting or removed.
    if (sclvar.isDerived() && (sclvar.derived instanceof SequenceLength)) {
      // Sequence length tests
      SequenceLength scl_seqlen = (SequenceLength) sclvar.derived;

    }

    // JHP: this presumes that this invariant is true and should thus be
    // moved to uninteresting or removed.
    {
      PptSlice1 seqslice = pptt.findSlice(seqvar);
      if (seqslice != null) {
        EltOneOfFloat eoo = EltOneOfFloat.find(seqslice);
        if ((eoo != null) && eoo.enoughSamples() && (eoo.num_elts() == 1)) {
          return new DiscardInfo(this, DiscardCode.obvious, "Obvious implied by " + eoo.format());
        }
      }
    }

    return null;
  }

  /**
   * Returns a list of non-instantiating suppressions for this invariant.
   */
  public NISuppressionSet get_ni_suppressions() {
    return (suppressions);
  }

  /** definition of this invariant (the suppressee) **/
  private static NISuppressee suppressee
    = new NISuppressee (SeqFloatGreaterEqual.class, 2);

  // suppressor definitions (used in suppressions below)
  private static NISuppressor v1_eq_v2
    = new NISuppressor (0, 1, SeqFloatEqual.class);
  private static NISuppressor v1_gt_v2
    = new NISuppressor (0, 1, SeqFloatGreaterThan.class);
  private static NISuppressor v1_lt_v2
    = new NISuppressor (0, 1, SeqFloatLessThan.class);

  // NI Suppressions for each type of comparison

    private static NISuppressionSet suppressions =
      new NISuppressionSet (new NISuppression[] {

        // v1 == v2 => v1 >= v2
        new NISuppression (v1_eq_v2, suppressee),

        // v1 > v2 => v1 >= v2
        new NISuppression (v1_gt_v2, suppressee),

      });

}
